@@ -1,4 +1,4 @@
-/* $Id: aircraft_gui.cpp 25500 2013-06-28 19:29:08Z rubidium $ */
+/* $Id: aircraft_gui.cpp 25454 2013-06-24 18:39:19Z rubidium $ */
 
 /*
  * This file is part of OpenTTD.
@@ -50,7 +50,7 @@ void DrawAircraftDetails(const Aircraft *v, int left, int right, int y)
 		}
 
 		if (u->cargo_cap != 0) {
-			uint cargo_count = u->cargo.Count();
+			uint cargo_count = u->cargo.StoredCount();
 
 			y_offset += FONT_HEIGHT_NORMAL + 1;
 			if (cargo_count != 0) {
