@@ -1,4 +1,4 @@
-/* $Id: company_cmd.cpp 25500 2013-06-28 19:29:08Z rubidium $ */
+/* $Id: company_cmd.cpp 26595 2014-05-18 11:21:59Z frosch $ */
 
 /*
  * This file is part of OpenTTD.
@@ -34,6 +34,8 @@
 #include "vehicle_func.h"
 #include "smallmap_gui.h"
 #include "game/game.hpp"
+#include "goal_base.h"
+#include "story_base.h"
 
 #include "table/strings.h"
 
@@ -82,6 +84,7 @@ void Company::PostDestructor(size_t index)
 	InvalidateWindowData(WC_GRAPH_LEGEND, 0, (int)index);
 	InvalidateWindowData(WC_PERFORMANCE_DETAIL, 0, (int)index);
 	InvalidateWindowData(WC_COMPANY_LEAGUE, 0, 0);
+	InvalidateWindowData(WC_LINKGRAPH_LEGEND, 0);
 	/* If the currently shown error message has this company in it, then close it. */
 	InvalidateWindowData(WC_ERRMSG, 0);
 }
@@ -559,6 +562,7 @@ Company *DoStartupNewCompany(bool is_ai, CompanyID company = INVALID_COMPANY)
 	SetWindowDirty(WC_GRAPH_LEGEND, 0);
 	SetWindowClassesDirty(WC_CLIENT_LIST_POPUP);
 	SetWindowDirty(WC_CLIENT_LIST, 0);
+	InvalidateWindowData(WC_LINKGRAPH_LEGEND, 0);
 	BuildOwnerLegend();
 	InvalidateWindowData(WC_SMALLMAP, 0, 1);
 
@@ -849,59 +853,21 @@ CommandCost CmdCompanyCtrl(TileIndex tile, DoCommandFlag flags, uint32 p1, uint3
 				MarkWholeScreenDirty();
 			}
 
-			if (_network_server) {
-				if (ci != NULL) {
-					/* ci is only NULL when replaying.
-					 * When replaying no client is actually in need of an update. */
-					ci->client_playas = c->index;
-					NetworkUpdateClientInfo(ci->client_id);
-				}
-
-				if (Company::IsValidID(c->index)) {
-					_network_company_states[c->index].months_empty = 0;
-					_network_company_states[c->index].password[0] = '\0';
-					NetworkServerUpdateCompanyPassworded(c->index, false);
-
-					/* XXX - When a client joins, we automatically set its name to the
-					 * client's name (for some reason). As it stands now only the server
-					 * knows the client's name, so it needs to send out a "broadcast" to
-					 * do this. To achieve this we send a network command. However, it
-					 * uses _local_company to execute the command as.  To prevent abuse
-					 * (eg. only yourself can change your name/company), we 'cheat' by
-					 * impersonation _local_company as the server. Not the best solution;
-					 * but it works.
-					 * TODO: Perhaps this could be improved by when the client is ready
-					 * with joining to let it send itself the command, and not the server?
-					 * For example in network_client.c:534? */
-					if (ci != NULL) {
-						/* ci is only NULL when replaying.
-						 * When replaying, the command to rename the president will
-						 * automatically be ran, so this is not even needed to get
-						 * the exact same state. */
-						NetworkSendCommand(0, 0, 0, CMD_RENAME_PRESIDENT, NULL, ci->client_name, c->index);
-					}
-				}
-
-				/* Announce new company on network. */
-				NetworkAdminCompanyInfo(c, true);
-
-				if (ci != NULL) {
-					/* ci is only NULL when replaying.
-					 * When replaying, the message that someone started a new company
-					 * is not interesting at all. */
-					NetworkServerSendChat(NETWORK_ACTION_COMPANY_NEW, DESTTYPE_BROADCAST, 0, "", ci->client_id, c->index + 1);
-				}
-			}
+			NetworkServerNewCompany(c, ci);
 #endif /* ENABLE_NETWORK */
 			break;
 		}
 
-		case 1: // Make a new AI company
+		case 1: { // Make a new AI company
 			if (!(flags & DC_EXEC)) return CommandCost();
 
 			if (company_id != INVALID_COMPANY && (company_id >= MAX_COMPANIES || Company::IsValidID(company_id))) return CMD_ERROR;
-			DoStartupNewCompany(true, company_id);
+			Company *c = DoStartupNewCompany(true, company_id);
+#ifdef ENABLE_NETWORK
+			if (c != NULL) NetworkServerNewCompany(c, NULL);
+#endif /* ENABLE_NETWORK */
 			break;
+		}
 
 		case 2: { // Delete a company
 			CompanyRemoveReason reason = (CompanyRemoveReason)GB(p2, 0, 2);
@@ -932,6 +898,8 @@ CommandCost CmdCompanyCtrl(TileIndex tile, DoCommandFlag flags, uint32 p1, uint3
 			AI::BroadcastNewEvent(new ScriptEventCompanyBankrupt(c_index));
 			Game::NewEvent(new ScriptEventCompanyBankrupt(c_index));
 			CompanyAdminRemove(c_index, (CompanyRemoveReason)reason);
+
+			if (StoryPage::GetNumItems() == 0 || Goal::GetNumItems() == 0) InvalidateWindowData(WC_MAIN_TOOLBAR, 0);
 			break;
 		}
 
@@ -1052,6 +1020,7 @@ CommandCost CmdSetCompanyColour(TileIndex tile, DoCommandFlag flags, uint32 p1,
 		InvalidateWindowData(WC_DELIVERED_CARGO, 0);
 		InvalidateWindowData(WC_PERFORMANCE_HISTORY, 0);
 		InvalidateWindowData(WC_COMPANY_VALUE, 0);
+		InvalidateWindowData(WC_LINKGRAPH_LEGEND, 0);
 		/* The smallmap owner view also stores the company colours. */
 		BuildOwnerLegend();
 		InvalidateWindowData(WC_SMALLMAP, 0, 1);
