@@ -1,4 +1,4 @@
-/* $Id: date_gui.cpp 24801 2012-12-08 17:18:51Z frosch $ */
+/* $Id: date_gui.cpp 26668 2014-06-29 17:58:58Z rubidium $ */
 
 /*
  * This file is part of OpenTTD.
@@ -38,22 +38,22 @@ struct SetDateWindow : Window {
 	 * @param max_year the maximum year (inclusive) to show in the year dropdown
 	 * @param callback the callback to call once a date has been selected
 	 */
-	SetDateWindow(const WindowDesc *desc, WindowNumber window_number, Window *parent, Date initial_date, Year min_year, Year max_year, SetDateCallback *callback) :
-			Window(),
+	SetDateWindow(WindowDesc *desc, WindowNumber window_number, Window *parent, Date initial_date, Year min_year, Year max_year, SetDateCallback *callback) :
+			Window(desc),
 			callback(callback),
 			min_year(max(MIN_YEAR, min_year)),
 			max_year(min(MAX_YEAR, max_year))
 	{
 		assert(this->min_year <= this->max_year);
 		this->parent = parent;
-		this->InitNested(desc, window_number);
+		this->InitNested(window_number);
 
 		if (initial_date == 0) initial_date = _date;
 		ConvertDateToYMD(initial_date, &this->date);
 		this->date.year = Clamp(this->date.year, min_year, max_year);
 	}
 
-	virtual Point OnInitialPosition(const WindowDesc *desc, int16 sm_width, int16 sm_height, int window_number)
+	virtual Point OnInitialPosition(int16 sm_width, int16 sm_height, int window_number)
 	{
 		Point pt = { this->parent->left + this->parent->width / 2 - sm_width / 2, this->parent->top + this->parent->height / 2 - sm_height / 2 };
 		return pt;
@@ -73,14 +73,14 @@ struct SetDateWindow : Window {
 
 			case WID_SD_DAY:
 				for (uint i = 0; i < 31; i++) {
-					list->push_back(new DropDownListStringItem(STR_ORDINAL_NUMBER_1ST + i, i + 1, false));
+					*list->Append() = new DropDownListStringItem(STR_DAY_NUMBER_1ST + i, i + 1, false);
 				}
 				selected = this->date.day;
 				break;
 
 			case WID_SD_MONTH:
 				for (uint i = 0; i < 12; i++) {
-					list->push_back(new DropDownListStringItem(STR_MONTH_JAN + i, i, false));
+					*list->Append() = new DropDownListStringItem(STR_MONTH_JAN + i, i, false);
 				}
 				selected = this->date.month;
 				break;
@@ -89,7 +89,7 @@ struct SetDateWindow : Window {
 				for (Year i = this->min_year; i <= this->max_year; i++) {
 					DropDownListParamStringItem *item = new DropDownListParamStringItem(STR_JUST_INT, i, false);
 					item->SetParam(0, i);
-					list->push_back(item);
+					*list->Append() = item;
 				}
 				selected = this->date.year;
 				break;
@@ -106,7 +106,7 @@ struct SetDateWindow : Window {
 
 			case WID_SD_DAY:
 				for (uint i = 0; i < 31; i++) {
-					d = maxdim(d, GetStringBoundingBox(STR_ORDINAL_NUMBER_1ST + i));
+					d = maxdim(d, GetStringBoundingBox(STR_DAY_NUMBER_1ST + i));
 				}
 				break;
 
@@ -130,7 +130,7 @@ struct SetDateWindow : Window {
 	virtual void SetStringParameters(int widget) const
 	{
 		switch (widget) {
-			case WID_SD_DAY:   SetDParam(0, this->date.day - 1 + STR_ORDINAL_NUMBER_1ST); break;
+			case WID_SD_DAY:   SetDParam(0, this->date.day - 1 + STR_DAY_NUMBER_1ST); break;
 			case WID_SD_MONTH: SetDParam(0, this->date.month + STR_MONTH_JAN); break;
 			case WID_SD_YEAR:  SetDParam(0, this->date.year); break;
 		}
@@ -146,7 +146,7 @@ struct SetDateWindow : Window {
 				break;
 
 			case WID_SD_SET_DATE:
-				if (this->callback != NULL) this->callback(this->parent, ConvertYMDToDate(this->date.year, this->date.month, this->date.day));
+				if (this->callback != NULL) this->callback(this, ConvertYMDToDate(this->date.year, this->date.month, this->date.day));
 				delete this;
 				break;
 		}
@@ -194,8 +194,8 @@ static const NWidgetPart _nested_set_date_widgets[] = {
 };
 
 /** Description of the date setting window. */
-static const WindowDesc _set_date_desc(
-	WDP_CENTER, 0, 0,
+static WindowDesc _set_date_desc(
+	WDP_CENTER, NULL, 0, 0,
 	WC_SET_DATE, WC_NONE,
 	0,
 	_nested_set_date_widgets, lengthof(_nested_set_date_widgets)
