@@ -1,4 +1,4 @@
-/* $Id: economy.cpp 25498 2013-06-28 19:21:24Z rubidium $ */
+/* $Id: economy.cpp 26972 2014-10-06 20:10:07Z frosch $ */
 
 /*
  * This file is part of OpenTTD.
@@ -19,6 +19,7 @@
 #include "network/network_func.h"
 #include "ai/ai.hpp"
 #include "aircraft.h"
+#include "train.h"
 #include "newgrf_engine.h"
 #include "engine_base.h"
 #include "ground_vehicle.hpp"
@@ -42,10 +43,13 @@
 #include "economy_base.h"
 #include "core/pool_func.hpp"
 #include "core/backup_type.hpp"
+#include "cargo_type.h"
 #include "water.h"
 #include "game/game.hpp"
 #include "cargomonitor.h"
 #include "goal_base.h"
+#include "story_base.h"
+#include "linkgraph/refresh.h"
 
 #include "table/strings.h"
 #include "table/pricebase.h"
@@ -509,7 +513,7 @@ void ChangeOwnershipOfCompanyItems(Owner old_owner, Owner new_owner)
 		if (si->owner == old_owner) si->owner = new_owner == INVALID_OWNER ? OWNER_NONE : new_owner;
 	}
 
-	/* Remove Game Script created Goals and CargoMonitors. */
+	/* Remove Game Script created Goals, CargoMonitors and Story pages. */
 	Goal *g;
 	FOR_ALL_GOALS(g) {
 		if (g->company == old_owner) delete g;
@@ -518,6 +522,11 @@ void ChangeOwnershipOfCompanyItems(Owner old_owner, Owner new_owner)
 	ClearCargoPickupMonitoring(old_owner);
 	ClearCargoDeliveryMonitoring(old_owner);
 
+	StoryPage *sp;
+	FOR_ALL_STORY_PAGES(sp) {
+		if (sp->company == old_owner) delete sp;
+	}
+
 	/* Change colour of existing windows */
 	if (new_owner != INVALID_OWNER) ChangeWindowOwner(old_owner, new_owner);
 
@@ -1058,9 +1067,9 @@ static Money DeliverGoods(int num_pieces, CargoID cargo_type, StationID dest, Ti
 
 	/* Update station statistics */
 	if (accepted > 0) {
-		SetBit(st->goods[cargo_type].acceptance_pickup, GoodsEntry::GES_EVER_ACCEPTED);
-		SetBit(st->goods[cargo_type].acceptance_pickup, GoodsEntry::GES_CURRENT_MONTH);
-		SetBit(st->goods[cargo_type].acceptance_pickup, GoodsEntry::GES_ACCEPTED_BIGTICK);
+		SetBit(st->goods[cargo_type].status, GoodsEntry::GES_EVER_ACCEPTED);
+		SetBit(st->goods[cargo_type].status, GoodsEntry::GES_CURRENT_MONTH);
+		SetBit(st->goods[cargo_type].status, GoodsEntry::GES_ACCEPTED_BIGTICK);
 	}
 
 	/* Update company statistics */
@@ -1140,21 +1149,23 @@ CargoPayment::~CargoPayment()
 
 	this->front->cargo_payment = NULL;
 
-	if (this->visual_profit == 0) return;
+	if (this->visual_profit == 0 && this->visual_transfer == 0) return;
 
 	Backup<CompanyByte> cur_company(_current_company, this->front->owner, FILE_LINE);
 
 	SubtractMoneyFromCompany(CommandCost(this->front->GetExpenseType(true), -this->route_profit));
-	this->front->profit_this_year += this->visual_profit << 8;
+	this->front->profit_this_year += (this->visual_profit + this->visual_transfer) << 8;
 
-	if (this->route_profit != 0) {
-		if (IsLocalCompany() && !PlayVehicleSound(this->front, VSE_LOAD_UNLOAD)) {
-			SndPlayVehicleFx(SND_14_CASHTILL, this->front);
-		}
+	if (this->route_profit != 0 && IsLocalCompany() && !PlayVehicleSound(this->front, VSE_LOAD_UNLOAD)) {
+		SndPlayVehicleFx(SND_14_CASHTILL, this->front);
+	}
 
-		ShowCostOrIncomeAnimation(this->front->x_pos, this->front->y_pos, this->front->z_pos, -this->visual_profit);
-	} else {
-		ShowFeederIncomeAnimation(this->front->x_pos, this->front->y_pos, this->front->z_pos, this->visual_profit);
+	if (this->visual_transfer != 0) {
+		ShowFeederIncomeAnimation(this->front->x_pos, this->front->y_pos,
+				this->front->z_pos, this->visual_transfer, -this->visual_profit);
+	} else if (this->visual_profit != 0) {
+		ShowCostOrIncomeAnimation(this->front->x_pos, this->front->y_pos,
+				this->front->z_pos, -this->visual_profit);
 	}
 
 	cur_company.Restore();
@@ -1176,7 +1187,7 @@ void CargoPayment::PayFinalDelivery(const CargoPacket *cp, uint count)
 	this->route_profit += profit;
 
 	/* The vehicle's profit is whatever route profit there is minus feeder shares. */
-	this->visual_profit += profit - cp->FeederShare();
+	this->visual_profit += profit - cp->FeederShare(count);
 }
 
 /**
@@ -1196,82 +1207,352 @@ Money CargoPayment::PayTransfer(const CargoPacket *cp, uint count)
 
 	profit = profit * _settings_game.economy.feeder_payment_share / 100;
 
-	this->visual_profit += profit; // accumulate transfer profits for whole vehicle
+	this->visual_transfer += profit; // accumulate transfer profits for whole vehicle
 	return profit; // account for the (virtual) profit already made for the cargo packet
 }
 
 /**
  * Prepare the vehicle to be unloaded.
+ * @param curr_station the station where the consist is at the moment
  * @param front_v the vehicle to be unloaded
  */
 void PrepareUnload(Vehicle *front_v)
 {
+	Station *curr_station = Station::Get(front_v->last_station_visited);
+	curr_station->loading_vehicles.push_back(front_v);
+
 	/* At this moment loading cannot be finished */
 	ClrBit(front_v->vehicle_flags, VF_LOADING_FINISHED);
 
-	/* Start unloading in at the first possible moment */
+	/* Start unloading at the first possible moment */
 	front_v->load_unload_ticks = 1;
 
-	if ((front_v->current_order.GetUnloadType() & OUFB_NO_UNLOAD) == 0) {
-		for (Vehicle *v = front_v; v != NULL; v = v->Next()) {
-			if (v->cargo_cap > 0 && !v->cargo.Empty()) {
-				SetBit(v->vehicle_flags, VF_CARGO_UNLOADING);
-			}
-		}
-	}
-
 	assert(front_v->cargo_payment == NULL);
 	/* One CargoPayment per vehicle and the vehicle limit equals the
 	 * limit in number of CargoPayments. Can't go wrong. */
 	assert_compile(CargoPaymentPool::MAX_SIZE == VehiclePool::MAX_SIZE);
 	assert(CargoPayment::CanAllocateItem());
 	front_v->cargo_payment = new CargoPayment(front_v);
+
+	StationIDStack next_station = front_v->GetNextStoppingStation();
+	if (front_v->orders.list == NULL || (front_v->current_order.GetUnloadType() & OUFB_NO_UNLOAD) == 0) {
+		Station *st = Station::Get(front_v->last_station_visited);
+		for (Vehicle *v = front_v; v != NULL; v = v->Next()) {
+			const GoodsEntry *ge = &st->goods[v->cargo_type];
+			if (v->cargo_cap > 0 && v->cargo.TotalCount() > 0) {
+				v->cargo.Stage(
+						HasBit(ge->status, GoodsEntry::GES_ACCEPTANCE),
+						front_v->last_station_visited, next_station,
+						front_v->current_order.GetUnloadType(), ge,
+						front_v->cargo_payment);
+				if (v->cargo.UnloadCount() > 0) SetBit(v->vehicle_flags, VF_CARGO_UNLOADING);
+			}
+		}
+	}
 }
 
 /**
- * Checks whether an articulated vehicle is empty.
- * @param v Vehicle
- * @return true if all parts are empty.
+ * Gets the amount of cargo the given vehicle can load in the current tick.
+ * This is only about loading speed. The free capacity is ignored.
+ * @param v Vehicle to be queried.
+ * @return Amount of cargo the vehicle can load at once.
  */
-static bool IsArticulatedVehicleEmpty(Vehicle *v)
+static uint GetLoadAmount(Vehicle *v)
 {
-	v = v->GetFirstEnginePart();
+	const Engine *e = v->GetEngine();
+	uint load_amount = e->info.load_amount;
 
-	for (; v != NULL; v = v->HasArticulatedPart() ? v->GetNextArticulatedPart() : NULL) {
-		if (v->cargo.Count() != 0) return false;
+	/* The default loadamount for mail is 1/4 of the load amount for passengers */
+	bool air_mail = v->type == VEH_AIRCRAFT && !Aircraft::From(v)->IsNormalAircraft();
+	if (air_mail) load_amount = CeilDiv(load_amount, 4);
+
+	if (_settings_game.order.gradual_loading) {
+		uint16 cb_load_amount = CALLBACK_FAILED;
+		if (e->GetGRF() != NULL && e->GetGRF()->grf_version >= 8) {
+			/* Use callback 36 */
+			cb_load_amount = GetVehicleProperty(v, PROP_VEHICLE_LOAD_AMOUNT, CALLBACK_FAILED);
+		} else if (HasBit(e->info.callback_mask, CBM_VEHICLE_LOAD_AMOUNT)) {
+			/* Use callback 12 */
+			cb_load_amount = GetVehicleCallback(CBID_VEHICLE_LOAD_AMOUNT, 0, 0, v->engine_type, v);
+		}
+		if (cb_load_amount != CALLBACK_FAILED) {
+			if (e->GetGRF()->grf_version < 8) cb_load_amount = GB(cb_load_amount, 0, 8);
+			if (cb_load_amount >= 0x100) {
+				ErrorUnknownCallbackResult(e->GetGRFID(), CBID_VEHICLE_LOAD_AMOUNT, cb_load_amount);
+			} else if (cb_load_amount != 0) {
+				load_amount = cb_load_amount;
+			}
+		}
 	}
 
+	/* Scale load amount the same as capacity */
+	if (HasBit(e->info.misc_flags, EF_NO_DEFAULT_CARGO_MULTIPLIER) && !air_mail) load_amount = CeilDiv(load_amount * CargoSpec::Get(v->cargo_type)->multiplier, 0x100);
+
+	return load_amount;
+}
+
+/**
+ * Reserves cargo if the full load order and improved_load is set or if the
+ * current order allows autorefit.
+ * @param st Station where the consist is loading at the moment.
+ * @param u Front of the loading vehicle consist.
+ * @param consist_capleft If given, save free capacities after reserving there.
+ * @param next_station Station(s) the vehicle will stop at next.
+ */
+static void ReserveConsist(Station *st, Vehicle *u, CargoArray *consist_capleft, StationIDStack next_station)
+{
+	Vehicle *next_cargo = u;
+	uint32 seen_cargos = 0;
+
+	while (next_cargo != NULL) {
+		if (next_cargo->cargo_cap == 0) {
+			/* No need to reserve for vehicles without capacity. */
+			next_cargo = next_cargo->Next();
+			continue;
+		}
+
+		CargoID current_cargo = next_cargo->cargo_type;
+
+		Vehicle *v = next_cargo;
+		SetBit(seen_cargos, current_cargo);
+		next_cargo = NULL;
+		for (; v != NULL; v = v->Next()) {
+			if (v->cargo_type != current_cargo) {
+				/* Save start point for next cargo type. */
+				if (next_cargo == NULL && !HasBit(seen_cargos, v->cargo_type)) next_cargo = v;
+				continue;
+			}
+
+			assert(v->cargo_cap >= v->cargo.RemainingCount());
+			uint cap = v->cargo_cap - v->cargo.RemainingCount();
+
+			/* Nothing to do if the vehicle is full */
+			if (cap > 0) {
+				cap -= st->goods[v->cargo_type].cargo.Reserve(cap, &v->cargo, st->xy, next_station);
+			}
+
+			if (consist_capleft != NULL) {
+				(*consist_capleft)[current_cargo] += cap;
+			}
+		}
+	}
+}
+
+/**
+ * Iterate the articulated parts of a vehicle, also considering the special cases of "normal"
+ * aircraft and double headed trains. Apply an action to each vehicle and immediately return false
+ * if that action does so. Otherwise return true.
+ * @tparam Taction Class of action to be applied. Must implement bool operator()([const] Vehicle *).
+ * @param v First articulated part.
+ * @param action Instance of Taction.
+ * @return false if any of the action invocations returned false, true otherwise.
+ */
+template<class Taction>
+bool IterateVehicleParts(Vehicle *v, Taction action)
+{
+	for (Vehicle *w = v; w != NULL;
+			w = w->HasArticulatedPart() ? w->GetNextArticulatedPart() : NULL) {
+		if (!action(w)) return false;
+		if (w->type == VEH_TRAIN) {
+			Train *train = Train::From(w);
+			if (train->IsMultiheaded() && !action(train->other_multiheaded_part)) return false;
+		}
+	}
+	if (v->type == VEH_AIRCRAFT && Aircraft::From(v)->IsNormalAircraft()) return action(v->Next());
 	return true;
 }
 
+/**
+ * Action to check if a vehicle has no stored cargo.
+ */
+struct IsEmptyAction
+{
+	/**
+	 * Checks if the vehicle has stored cargo.
+	 * @param v Vehicle to be checked.
+	 * @return true if v is either empty or has only reserved cargo, false otherwise.
+	 */
+	bool operator()(const Vehicle *v)
+	{
+		return v->cargo.StoredCount() == 0;
+	}
+};
+
+/**
+ * Refit preparation action.
+ */
+struct PrepareRefitAction
+{
+	CargoArray &consist_capleft; ///< Capacities left in the consist.
+	uint32 &refit_mask;          ///< Bitmask of possible refit cargoes.
+
+	/**
+	 * Create a refit preparation action.
+	 * @param consist_capleft Capacities left in consist, to be updated here.
+	 * @param refit_mask Refit mask to be constructed from refit information of vehicles.
+	 */
+	PrepareRefitAction(CargoArray &consist_capleft, uint32 &refit_mask) :
+		consist_capleft(consist_capleft), refit_mask(refit_mask) {}
+
+	/**
+	 * Prepares for refitting of a vehicle, subtracting its free capacity from consist_capleft and
+	 * adding the cargoes it can refit to to the refit mask.
+	 * @param v The vehicle to be refitted.
+	 * @return true.
+	 */
+	bool operator()(const Vehicle *v)
+	{
+		this->consist_capleft[v->cargo_type] -= v->cargo_cap - v->cargo.ReservedCount();
+		this->refit_mask |= EngInfo(v->engine_type)->refit_mask;
+		return true;
+	}
+};
+
+/**
+ * Action for returning reserved cargo.
+ */
+struct ReturnCargoAction
+{
+	Station *st;        ///< Station to give the returned cargo to.
+	StationID next_hop; ///< Next hop the cargo should be assigned to.
+
+	/**
+	 * Construct a cargo return action.
+	 * @param st Station to give the returned cargo to.
+	 * @param next_one Next hop the cargo should be assigned to.
+	 */
+	ReturnCargoAction(Station *st, StationID next_one) : st(st), next_hop(next_one) {}
+
+	/**
+	 * Return all reserved cargo from a vehicle.
+	 * @param v Vehicle to return cargo from.
+	 * @return true.
+	 */
+	bool operator()(Vehicle *v)
+	{
+		v->cargo.Return(UINT_MAX, &this->st->goods[v->cargo_type].cargo, this->next_hop);
+		return true;
+	}
+};
+
+/**
+ * Action for finalizing a refit.
+ */
+struct FinalizeRefitAction
+{
+	CargoArray &consist_capleft;  ///< Capacities left in the consist.
+	Station *st;                  ///< Station to reserve cargo from.
+	StationIDStack &next_station; ///< Next hops to reserve cargo for.
+
+	/**
+	 * Create a finalizing action.
+	 * @param consist_capleft Capacities left in the consist.
+	 * @param st Station to reserve cargo from.
+	 * @param next_station Next hops to reserve cargo for.
+	 */
+	FinalizeRefitAction(CargoArray &consist_capleft, Station *st, StationIDStack &next_station) :
+		consist_capleft(consist_capleft), st(st), next_station(next_station) {}
+
+	/**
+	 * Reserve cargo from the station and update the remaining consist capacities with the
+	 * vehicle's remaining free capacity.
+	 * @param v Vehicle to be finalized.
+	 * @return true.
+	 */
+	bool operator()(Vehicle *v)
+	{
+		this->st->goods[v->cargo_type].cargo.Reserve(v->cargo_cap - v->cargo.RemainingCount(),
+				&v->cargo, st->xy, next_station);
+		this->consist_capleft[v->cargo_type] += v->cargo_cap - v->cargo.RemainingCount();
+		return true;
+	}
+};
+
+/**
+ * Refit a vehicle in a station.
+ * @param v Vehicle to be refitted.
+ * @param consist_capleft Added cargo capacities in the consist.
+ * @param st Station the vehicle is loading at.
+ * @param next_station Possible next stations the vehicle can travel to.
+ * @param new_cid Target cargo for refit.
+ */
+static void HandleStationRefit(Vehicle *v, CargoArray &consist_capleft, Station *st, StationIDStack next_station, CargoID new_cid)
+{
+	Vehicle *v_start = v->GetFirstEnginePart();
+	if (!IterateVehicleParts(v_start, IsEmptyAction())) return;
+
+	Backup<CompanyByte> cur_company(_current_company, v->owner, FILE_LINE);
+
+	uint32 refit_mask = v->GetEngine()->info.refit_mask;
+
+	/* Remove old capacity from consist capacity and collect refit mask. */
+	IterateVehicleParts(v_start, PrepareRefitAction(consist_capleft, refit_mask));
+
+	if (new_cid == CT_AUTO_REFIT) {
+		/* Get a refittable cargo type with waiting cargo for next_station or INVALID_STATION. */
+		CargoID cid;
+		new_cid = v_start->cargo_type;
+		FOR_EACH_SET_CARGO_ID(cid, refit_mask) {
+			if (st->goods[cid].cargo.HasCargoFor(next_station)) {
+				/* Try to find out if auto-refitting would succeed. In case the refit is allowed,
+				 * the returned refit capacity will be greater than zero. */
+				DoCommand(v_start->tile, v_start->index, cid | 1U << 6 | 0xFF << 8 | 1U << 16, DC_QUERY_COST, GetCmdRefitVeh(v_start)); // Auto-refit and only this vehicle including artic parts.
+				/* Try to balance different loadable cargoes between parts of the consist, so that
+				 * all of them can be loaded. Avoid a situation where all vehicles suddenly switch
+				 * to the first loadable cargo for which there is only one packet. If the capacities
+				 * are equal refit to the cargo of which most is available. This is important for
+				 * consists of only a single vehicle as those will generally have a consist_capleft
+				 * of 0 for all cargoes. */
+				if (_returned_refit_capacity > 0 && (consist_capleft[cid] < consist_capleft[new_cid] ||
+						(consist_capleft[cid] == consist_capleft[new_cid] &&
+						st->goods[cid].cargo.AvailableCount() > st->goods[new_cid].cargo.AvailableCount()))) {
+					new_cid = cid;
+				}
+			}
+		}
+	}
+
+	/* Refit if given a valid cargo. */
+	if (new_cid < NUM_CARGO && new_cid != v_start->cargo_type) {
+		/* INVALID_STATION because in the DT_MANUAL case that's correct and in the DT_(A)SYMMETRIC
+		 * cases the next hop of the vehicle doesn't really tell us anything if the cargo had been
+		 * "via any station" before reserving. We rather produce some more "any station" cargo than
+		 * misrouting it. */
+		IterateVehicleParts(v_start, ReturnCargoAction(st, INVALID_STATION));
+		CommandCost cost = DoCommand(v_start->tile, v_start->index, new_cid | 1U << 6 | 0xFF << 8 | 1U << 16, DC_EXEC, GetCmdRefitVeh(v_start)); // Auto-refit and only this vehicle including artic parts.
+		if (cost.Succeeded()) v->First()->profit_this_year -= cost.GetCost() << 8;
+	}
+
+	/* Add new capacity to consist capacity and reserve cargo */
+	IterateVehicleParts(v_start, FinalizeRefitAction(consist_capleft, st, next_station));
+
+	cur_company.Restore();
+}
+
 /**
  * Loads/unload the vehicle if possible.
  * @param front the vehicle to be (un)loaded
- * @param cargo_left the amount of each cargo type that is
- *                   virtually left on the platform to be
- *                   picked up by another vehicle when all
- *                   previous vehicles have loaded.
  */
-static void LoadUnloadVehicle(Vehicle *front, int *cargo_left)
+static void LoadUnloadVehicle(Vehicle *front)
 {
 	assert(front->current_order.IsType(OT_LOADING));
 
-	/* We have not waited enough time till the next round of loading/unloading */
-	if (front->load_unload_ticks != 0) {
-		if (_settings_game.order.improved_load && (front->current_order.GetLoadType() & OLFB_FULL_LOAD)) {
-			/* 'Reserve' this cargo for this vehicle, because we were first. */
-			for (Vehicle *v = front; v != NULL; v = v->Next()) {
-				int cap_left = v->cargo_cap;
-				if (!HasBit(v->vehicle_flags, VF_CARGO_UNLOADING)) cap_left -= v->cargo.Count();
-				if (cap_left > 0) cargo_left[v->cargo_type] -= cap_left;
-			}
-		}
-		return;
-	}
-
 	StationID last_visited = front->last_station_visited;
 	Station *st = Station::Get(last_visited);
 
+	StationIDStack next_station = front->GetNextStoppingStation();
+	bool use_autorefit = front->current_order.IsRefit() && front->current_order.GetRefitCargo() == CT_AUTO_REFIT;
+	CargoArray consist_capleft;
+	if (_settings_game.order.improved_load &&
+			((front->current_order.GetLoadType() & OLFB_FULL_LOAD) != 0 || use_autorefit)) {
+		ReserveConsist(st, front,
+				(use_autorefit && front->load_unload_ticks != 0) ? &consist_capleft : NULL,
+				next_station);
+	}
+
+	/* We have not waited enough time till the next round of loading/unloading */
+	if (front->load_unload_ticks != 0) return;
+
 	if (front->type == VEH_TRAIN && (!IsTileType(front->tile, MP_STATION) || GetStationIndex(front->tile) != st->index)) {
 		/* The train reversed in the station. Take the "easy" way
 		 * out and let the train just leave as it always did. */
@@ -1280,16 +1561,6 @@ static void LoadUnloadVehicle(Vehicle *front, int *cargo_left)
 		return;
 	}
 
-	bool use_autorefit = front->current_order.IsRefit() && front->current_order.GetRefitCargo() == CT_AUTO_REFIT;
-	CargoArray consist_capleft;
-	if (use_autorefit) {
-		/* Sum cargo, that can be loaded without refitting */
-		for (Vehicle *v = front; v != NULL; v = v->Next()) {
-			int cap_left = v->cargo_cap - v->cargo.Count();
-			if (cap_left > 0) consist_capleft[v->cargo_type] += cap_left;
-		}
-	}
-
 	int unloading_time = 0;
 	bool dirty_vehicle = false;
 	bool dirty_station = false;
@@ -1312,85 +1583,65 @@ static void LoadUnloadVehicle(Vehicle *front, int *cargo_left)
 		if (v->cargo_cap == 0) continue;
 		artic_part++;
 
-		const Engine *e = v->GetEngine();
-		uint load_amount = e->info.load_amount;
-
-		/* The default loadamount for mail is 1/4 of the load amount for passengers */
-		bool air_mail = v->type == VEH_AIRCRAFT && !Aircraft::From(v)->IsNormalAircraft();
-		if (air_mail) load_amount = CeilDiv(load_amount, 4);
-
-		if (_settings_game.order.gradual_loading) {
-			uint16 cb_load_amount = CALLBACK_FAILED;
-			if (e->GetGRF() != NULL && e->GetGRF()->grf_version >= 8) {
-				/* Use callback 36 */
-				cb_load_amount = GetVehicleProperty(v, PROP_VEHICLE_LOAD_AMOUNT, CALLBACK_FAILED);
-			} else if (HasBit(e->info.callback_mask, CBM_VEHICLE_LOAD_AMOUNT)) {
-				/* Use callback 12 */
-				cb_load_amount = GetVehicleCallback(CBID_VEHICLE_LOAD_AMOUNT, 0, 0, v->engine_type, v);
-			}
-			if (cb_load_amount != CALLBACK_FAILED) {
-				if (e->GetGRF()->grf_version < 8) cb_load_amount = GB(cb_load_amount, 0, 8);
-				if (cb_load_amount >= 0x100) {
-					ErrorUnknownCallbackResult(e->GetGRFID(), CBID_VEHICLE_LOAD_AMOUNT, cb_load_amount);
-				} else if (cb_load_amount != 0) {
-					load_amount = cb_load_amount;
-				}
-			}
-		}
-
-		/* Scale load amount the same as capacity */
-		if (HasBit(e->info.misc_flags, EF_NO_DEFAULT_CARGO_MULTIPLIER) && !air_mail) load_amount = CeilDiv(load_amount * CargoSpec::Get(v->cargo_type)->multiplier, 0x100);
+		uint load_amount = GetLoadAmount(v);
 
 		GoodsEntry *ge = &st->goods[v->cargo_type];
 
 		if (HasBit(v->vehicle_flags, VF_CARGO_UNLOADING) && (front->current_order.GetUnloadType() & OUFB_NO_UNLOAD) == 0) {
-			uint cargo_count = v->cargo.Count();
+			uint cargo_count = v->cargo.UnloadCount();
 			uint amount_unloaded = _settings_game.order.gradual_loading ? min(cargo_count, load_amount) : cargo_count;
 			bool remaining = false; // Are there cargo entities in this vehicle that can still be unloaded here?
-			bool accepted  = false; // Is the cargo accepted by the station?
 
 			payment->SetCargo(v->cargo_type);
 
-			if (HasBit(ge->acceptance_pickup, GoodsEntry::GES_ACCEPTANCE) && !(front->current_order.GetUnloadType() & OUFB_TRANSFER)) {
-				/* The cargo has reached its final destination, the packets may now be destroyed */
-				remaining = v->cargo.MoveTo<StationCargoList>(NULL, amount_unloaded, VehicleCargoList::MTA_FINAL_DELIVERY, payment, last_visited);
+			if (!HasBit(ge->status, GoodsEntry::GES_ACCEPTANCE) && v->cargo.ActionCount(VehicleCargoList::MTA_DELIVER) > 0) {
+				/* The station does not accept our goods anymore. */
+				if (front->current_order.GetUnloadType() & (OUFB_TRANSFER | OUFB_UNLOAD)) {
+					/* Transfer instead of delivering. */
+					v->cargo.Reassign<VehicleCargoList::MTA_DELIVER, VehicleCargoList::MTA_TRANSFER>(
+							v->cargo.ActionCount(VehicleCargoList::MTA_DELIVER), INVALID_STATION);
+				} else {
+					uint new_remaining = v->cargo.RemainingCount() + v->cargo.ActionCount(VehicleCargoList::MTA_DELIVER);
+					if (v->cargo_cap < new_remaining) {
+						/* Return some of the reserved cargo to not overload the vehicle. */
+						v->cargo.Return(new_remaining - v->cargo_cap, &ge->cargo, INVALID_STATION);
+					}
 
-				dirty_vehicle = true;
-				accepted = true;
-			}
+					/* Keep instead of delivering. This may lead to no cargo being unloaded, so ...*/
+					v->cargo.Reassign<VehicleCargoList::MTA_DELIVER, VehicleCargoList::MTA_KEEP>(
+							v->cargo.ActionCount(VehicleCargoList::MTA_DELIVER));
 
-			/* The !accepted || v->cargo.Count == cargo_count clause is there
-			 * to make it possible to force unload vehicles at the station where
-			 * they were loaded, but to not force unload the vehicle when the
-			 * station is still accepting the cargo in the vehicle. It doesn't
-			 * accept cargo that was loaded at the same station. */
-			if ((front->current_order.GetUnloadType() & (OUFB_UNLOAD | OUFB_TRANSFER)) && (!accepted || v->cargo.Count() == cargo_count)) {
-				remaining = v->cargo.MoveTo(&ge->cargo, amount_unloaded, front->current_order.GetUnloadType() & OUFB_TRANSFER ? VehicleCargoList::MTA_TRANSFER : VehicleCargoList::MTA_UNLOAD, payment);
-				if (!HasBit(ge->acceptance_pickup, GoodsEntry::GES_PICKUP)) {
-					SetBit(ge->acceptance_pickup, GoodsEntry::GES_PICKUP);
-					InvalidateWindowData(WC_STATION_LIST, last_visited);
+					/* ... say we unloaded something, otherwise we'll think we didn't unload
+					 * something and we didn't load something, so we must be finished
+					 * at this station. Setting the unloaded means that we will get a
+					 * retry for loading in the next cycle. */
+					anything_unloaded = true;
 				}
-
-				dirty_vehicle = dirty_station = true;
-			} else if (!accepted) {
-				/* The order changed while unloading (unset unload/transfer) or the
-				 * station does not accept our goods. */
-				ClrBit(v->vehicle_flags, VF_CARGO_UNLOADING);
-
-				/* Say we loaded something, otherwise we'll think we didn't unload
-				 * something and we didn't load something, so we must be finished
-				 * at this station. Setting the unloaded means that we will get a
-				 * retry for loading in the next cycle. */
-				anything_unloaded = true;
-				continue;
 			}
 
-			/* Deliver goods to the station */
-			st->time_since_unload = 0;
+			if (v->cargo.ActionCount(VehicleCargoList::MTA_TRANSFER) > 0) {
+				/* Mark the station dirty if we transfer, but not if we only deliver. */
+				dirty_station = true;
 
-			unloading_time += amount_unloaded;
+				if (!ge->HasRating()) {
+					/* Upon transfering cargo, make sure the station has a rating. Fake a pickup for the
+					 * first unload to prevent the cargo from quickly decaying after the initial drop. */
+					ge->time_since_pickup = 0;
+					SetBit(ge->status, GoodsEntry::GES_RATING);
+				}
+			}
+
+			amount_unloaded = v->cargo.Unload(amount_unloaded, &ge->cargo, payment);
+			remaining = v->cargo.UnloadCount() > 0;
+			if (amount_unloaded > 0) {
+				dirty_vehicle = true;
+				anything_unloaded = true;
+				unloading_time += amount_unloaded;
+
+				/* Deliver goods to the station */
+				st->time_since_unload = 0;
+			}
 
-			anything_unloaded = true;
 			if (_settings_game.order.gradual_loading && remaining) {
 				completely_emptied = false;
 			} else {
@@ -1405,67 +1656,14 @@ static void LoadUnloadVehicle(Vehicle *front, int *cargo_left)
 		if (front->current_order.GetLoadType() & OLFB_NO_LOAD || HasBit(front->vehicle_flags, VF_STOP_LOADING)) continue;
 
 		/* This order has a refit, if this is the first vehicle part carrying cargo and the whole vehicle is empty, try refitting. */
-		if (front->current_order.IsRefit() && artic_part == 1 && IsArticulatedVehicleEmpty(v) &&
-				(v->type != VEH_AIRCRAFT || (Aircraft::From(v)->IsNormalAircraft() && v->Next()->cargo.Count() == 0))) {
-			Vehicle *v_start = v->GetFirstEnginePart();
-			CargoID new_cid = front->current_order.GetRefitCargo();
-			byte new_subtype = front->current_order.GetRefitSubtype();
-
-			/* Remove old capacity from consist capacity */
-			consist_capleft[v_start->cargo_type] -= v_start->cargo_cap;
-			for (Vehicle *w = v_start; w->HasArticulatedPart(); ) {
-				w = w->GetNextArticulatedPart();
-				consist_capleft[w->cargo_type] -= w->cargo_cap;
-			}
-
-			Backup<CompanyByte> cur_company(_current_company, front->owner, FILE_LINE);
-
-			/* Check if all articulated parts are empty and collect refit mask. */
-			uint32 refit_mask = e->info.refit_mask;
-			Vehicle *w = v_start;
-			while (w->HasArticulatedPart()) {
-				w = w->GetNextArticulatedPart();
-				if (w->cargo.Count() > 0) new_cid = CT_NO_REFIT;
-				refit_mask |= EngInfo(w->engine_type)->refit_mask;
-			}
-
-			if (new_cid == CT_AUTO_REFIT) {
-				/* Get refittable cargo type with the most waiting cargo. */
-				int amount = 0;
-				CargoID cid;
-				FOR_EACH_SET_CARGO_ID(cid, refit_mask) {
-					/* Consider refitting to this cargo, if other vehicles of the consist cannot
-					 * already take the cargo without refitting */
-					if (cargo_left[cid] > (int)consist_capleft[cid] + amount) {
-						/* Try to find out if auto-refitting would succeed. In case the refit is allowed,
-						 * the returned refit capacity will be greater than zero. */
-						new_subtype = GetBestFittingSubType(v, v, cid);
-						DoCommand(v_start->tile, v_start->index, cid | 1U << 6 | new_subtype << 8 | 1U << 16, DC_QUERY_COST, GetCmdRefitVeh(v_start)); // Auto-refit and only this vehicle including artic parts.
-						if (_returned_refit_capacity > 0) {
-							amount = cargo_left[cid] - consist_capleft[cid];
-							new_cid = cid;
-						}
-					}
-				}
-			}
-
-			/* Refit if given a valid cargo. */
-			if (new_cid < NUM_CARGO) {
-				CommandCost cost = DoCommand(v_start->tile, v_start->index, new_cid | 1U << 6 | new_subtype << 8 | 1U << 16, DC_EXEC, GetCmdRefitVeh(v_start)); // Auto-refit and only this vehicle including artic parts.
-				if (cost.Succeeded()) front->profit_this_year -= cost.GetCost() << 8;
-				ge = &st->goods[v->cargo_type];
-			}
-
-			/* Add new capacity to consist capacity */
-			consist_capleft[v_start->cargo_type] += v_start->cargo_cap;
-			for (Vehicle *w = v_start; w->HasArticulatedPart(); ) {
-				w = w->GetNextArticulatedPart();
-				consist_capleft[w->cargo_type] += w->cargo_cap;
-			}
-
-			cur_company.Restore();
+		if (front->current_order.IsRefit() && artic_part == 1) {
+			HandleStationRefit(v, consist_capleft, st, next_station, front->current_order.GetRefitCargo());
+			ge = &st->goods[v->cargo_type];
 		}
 
+		/* As we're loading here the following link can carry the full capacity of the vehicle. */
+		v->refit_cap = v->cargo_cap;
+
 		/* update stats */
 		int t;
 		switch (front->type) {
@@ -1490,56 +1688,28 @@ static void LoadUnloadVehicle(Vehicle *front, int *cargo_left)
 		ge->last_age = min(_cur_year - front->build_year, 255);
 		ge->time_since_pickup = 0;
 
+		assert(v->cargo_cap >= v->cargo.StoredCount());
 		/* If there's goods waiting at the station, and the vehicle
 		 * has capacity for it, load it on the vehicle. */
-		int cap_left = v->cargo_cap - v->cargo.Count();
-		if (!ge->cargo.Empty() && cap_left > 0) {
-			uint cap = cap_left;
-			uint count = ge->cargo.Count();
+		uint cap_left = v->cargo_cap - v->cargo.StoredCount();
+		if (cap_left > 0 && (v->cargo.ActionCount(VehicleCargoList::MTA_LOAD) > 0 || ge->cargo.AvailableCount() > 0)) {
+			if (_settings_game.order.gradual_loading) cap_left = min(cap_left, load_amount);
+			if (v->cargo.StoredCount() == 0) TriggerVehicle(v, VEHICLE_TRIGGER_NEW_CARGO);
 
-			/* Skip loading this vehicle if another train/vehicle is already handling
-			 * the same cargo type at this station */
-			if (_settings_game.order.improved_load && cargo_left[v->cargo_type] <= 0) {
-				SetBit(cargo_not_full, v->cargo_type);
-				continue;
-			}
-
-			if (cap > count) cap = count;
-			if (_settings_game.order.gradual_loading) {
-				cap = min(cap, load_amount);
-				cap_left = min(cap_left, load_amount);
-			}
-			if (_settings_game.order.improved_load) {
-				/* Don't load stuff that is already 'reserved' for other vehicles */
-				cap = min((uint)cargo_left[v->cargo_type], cap);
-				count = cargo_left[v->cargo_type];
-				if (use_autorefit) {
-					/* When using autorefit, reserve all cargo for this wagon to prevent other wagons
-					 * from feeling the need to refit. */
-					uint total_cap_left = v->cargo_cap - v->cargo.Count();
-					cargo_left[v->cargo_type] -= total_cap_left;
-					consist_capleft[v->cargo_type] -= total_cap_left;
-					if (total_cap_left > cap && count > cap) {
-						/* Remember if there are reservations left so that we don't stop
-						 * loading before they're loaded. */
-						SetBit(reservation_left, v->cargo_type);
-					}
-				} else {
-					/* Update cargo left; but don't reserve everything yet, so other wagons
-					 * of the same consist load in parallel. */
-					cargo_left[v->cargo_type] -= cap;
-				}
+			uint loaded = ge->cargo.Load(cap_left, &v->cargo, st->xy, next_station);
+			if (v->cargo.ActionCount(VehicleCargoList::MTA_LOAD) > 0) {
+				/* Remember if there are reservations left so that we don't stop
+				 * loading before they're loaded. */
+				SetBit(reservation_left, v->cargo_type);
 			}
 
 			/* Store whether the maximum possible load amount was loaded or not.*/
-			if (count >= (uint)cap_left) {
+			if (loaded == cap_left) {
 				SetBit(full_load_amount, v->cargo_type);
 			} else {
 				ClrBit(full_load_amount, v->cargo_type);
 			}
 
-			if (v->cargo.Empty()) TriggerVehicle(v, VEHICLE_TRIGGER_NEW_CARGO);
-
 			/* TODO: Regarding this, when we do gradual loading, we
 			 * should first unload all vehicles and then start
 			 * loading them. Since this will cause
@@ -1547,26 +1717,26 @@ static void LoadUnloadVehicle(Vehicle *front, int *cargo_left)
 			 * the whole vehicle chain is really totally empty, the
 			 * completely_emptied assignment can then be safely
 			 * removed; that's how TTDPatch behaves too. --pasky */
-			completely_emptied = false;
-			anything_loaded = true;
+			if (loaded > 0) {
+				completely_emptied = false;
+				anything_loaded = true;
 
-			ge->cargo.MoveTo(&v->cargo, cap, StationCargoList::MTA_CARGO_LOAD, NULL, st->xy);
+				st->time_since_load = 0;
+				st->last_vehicle_type = v->type;
 
-			st->time_since_load = 0;
-			st->last_vehicle_type = v->type;
+				if (ge->cargo.TotalCount() == 0) {
+					TriggerStationRandomisation(st, st->xy, SRT_CARGO_TAKEN, v->cargo_type);
+					TriggerStationAnimation(st, st->xy, SAT_CARGO_TAKEN, v->cargo_type);
+					AirportAnimationTrigger(st, AAT_STATION_CARGO_TAKEN, v->cargo_type);
+				}
 
-			if (ge->cargo.Empty()) {
-				TriggerStationRandomisation(st, st->xy, SRT_CARGO_TAKEN, v->cargo_type);
-				TriggerStationAnimation(st, st->xy, SAT_CARGO_TAKEN, v->cargo_type);
-				AirportAnimationTrigger(st, AAT_STATION_CARGO_TAKEN, v->cargo_type);
+				unloading_time += loaded;
+
+				dirty_vehicle = dirty_station = true;
 			}
-
-			unloading_time += cap;
-
-			dirty_vehicle = dirty_station = true;
 		}
 
-		if (v->cargo.Count() >= v->cargo_cap) {
+		if (v->cargo.StoredCount() >= v->cargo_cap) {
 			SetBit(cargo_full, v->cargo_type);
 		} else {
 			SetBit(cargo_not_full, v->cargo_type);
@@ -1583,27 +1753,6 @@ static void LoadUnloadVehicle(Vehicle *front, int *cargo_left)
 	/* Only set completely_emptied, if we just unloaded all remaining cargo */
 	completely_emptied &= anything_unloaded;
 
-	/* For consists without autorefit-order we adjust the reserved cargo at the station after loading,
-	 * so that all wagons start loading if the consist is the first consist.
-	 *
-	 * If we use autorefit otoh, we only want to load/refit a vehicle if the other wagons cannot already hold the cargo,
-	 * to keep the option to still refit the vehicle when new cargo of different type shows up.
-	 */
-	if (_settings_game.order.improved_load && (front->current_order.GetLoadType() & OLFB_FULL_LOAD)) {
-		/* Update left cargo */
-		for (Vehicle *v = front; v != NULL; v = v->Next()) {
-			int cap_left = v->cargo_cap;
-			if (!HasBit(v->vehicle_flags, VF_CARGO_UNLOADING)) {
-				if (use_autorefit) {
-					continue;
-				} else {
-					cap_left -= v->cargo.Count();
-				}
-			}
-			if (cap_left > 0) cargo_left[v->cargo_type] -= cap_left;
-		}
-	}
-
 	if (!anything_unloaded) delete payment;
 
 	ClrBit(front->vehicle_flags, VF_STOP_LOADING);
@@ -1627,13 +1776,20 @@ static void LoadUnloadVehicle(Vehicle *front, int *cargo_left)
 			if (front->current_order.GetLoadType() == OLF_FULL_LOAD_ANY) {
 				/* if the aircraft carries passengers and is NOT full, then
 				 * continue loading, no matter how much mail is in */
-				if ((front->type == VEH_AIRCRAFT && IsCargoInClass(front->cargo_type, CC_PASSENGERS) && front->cargo_cap > front->cargo.Count()) ||
+				if ((front->type == VEH_AIRCRAFT && IsCargoInClass(front->cargo_type, CC_PASSENGERS) && front->cargo_cap > front->cargo.StoredCount()) ||
 						(cargo_not_full && (cargo_full & ~cargo_not_full) == 0)) { // There are still non-full cargoes
 					finished_loading = false;
 				}
 			} else if (cargo_not_full != 0) {
 				finished_loading = false;
 			}
+
+			/* Refresh next hop stats if we're full loading to make the links
+			 * known to the distribution algorithm and allow cargo to be sent
+			 * along them. Otherwise the vehicle could wait for cargo
+			 * indefinitely if it hasn't visited the other links yet, or if the
+			 * links die while it's loading. */
+			if (!finished_loading) LinkRefresher::Run(front);
 		}
 		unloading_time = 20;
 
@@ -1683,6 +1839,7 @@ static void LoadUnloadVehicle(Vehicle *front, int *cargo_left)
 	if (dirty_station) {
 		st->MarkTilesDirty(true);
 		SetWindowDirty(WC_STATION_VIEW, last_visited);
+		InvalidateWindowData(WC_STATION_LIST, last_visited);
 	}
 }
 
@@ -1718,13 +1875,9 @@ void LoadUnloadStation(Station *st)
 	 */
 	if (last_loading == NULL) return;
 
-	int cargo_left[NUM_CARGO];
-
-	for (uint i = 0; i < NUM_CARGO; i++) cargo_left[i] = st->goods[i].cargo.Count();
-
 	for (iter = st->loading_vehicles.begin(); iter != st->loading_vehicles.end(); ++iter) {
 		Vehicle *v = *iter;
-		if (!(v->vehstatus & (VS_STOPPED | VS_CRASHED))) LoadUnloadVehicle(v, cargo_left);
+		if (!(v->vehstatus & (VS_STOPPED | VS_CRASHED))) LoadUnloadVehicle(v);
 		if (v == last_loading) break;
 	}
 
