@@ -1,4 +1,4 @@
-/* $Id: fontcache.cpp 25987 2013-11-13 21:53:40Z rubidium $ */
+/* $Id: fontcache.cpp 26389 2014-03-03 21:34:36Z fonsinchen $ */
 
 /*
  * This file is part of OpenTTD.
@@ -81,6 +81,7 @@ public:
 	virtual bool GetDrawGlyphShadow();
 	virtual GlyphID MapCharToGlyph(WChar key) { assert(IsPrintable(key)); return SPRITE_GLYPH | key; }
 	virtual const void *GetFontTable(uint32 tag, size_t &length) { length = 0; return NULL; }
+	virtual const char *GetFontName() { return "sprite"; }
 };
 
 /**
@@ -239,6 +240,7 @@ public:
 	virtual bool GetDrawGlyphShadow();
 	virtual GlyphID MapCharToGlyph(WChar key);
 	virtual const void *GetFontTable(uint32 tag, size_t &length);
+	virtual const char *GetFontName() { return face->family_name; }
 };
 
 FT_Library _library = NULL;
@@ -272,23 +274,35 @@ FreeTypeFontCache::FreeTypeFontCache(FontSize fs, FT_Face face, int pixels) : Fo
 	}
 
 	FT_Error err = FT_Set_Pixel_Sizes(this->face, 0, pixels);
-	if (err == FT_Err_Invalid_Pixel_Size) {
+	if (err != FT_Err_Ok) {
 
 		/* Find nearest size to that requested */
 		FT_Bitmap_Size *bs = this->face->available_sizes;
 		int i = this->face->num_fixed_sizes;
-		int n = bs->height;
-		for (; --i; bs++) {
-			if (abs(pixels - bs->height) < abs(pixels - n)) n = bs->height;
-		}
+		if (i > 0) { // In pathetic cases one might get no fixed sizes at all.
+			int n = bs->height;
+			FT_Int chosen = 0;
+			for (; --i; bs++) {
+				if (abs(pixels - bs->height) >= abs(pixels - n)) continue;
+				n = bs->height;
+				chosen = this->face->num_fixed_sizes - i;
+			}
 
-		FT_Set_Pixel_Sizes(this->face, 0, n);
+			/* Don't use FT_Set_Pixel_Sizes here - it might give us another
+			 * error, even though the size is available (FS#5885). */
+			err = FT_Select_Size(this->face, chosen);
+		}
 	}
 
-	this->units_per_em = this->face->units_per_EM;
-	this->ascender     = this->face->size->metrics.ascender >> 6;
-	this->descender    = this->face->size->metrics.descender >> 6;
-	this->height       = this->ascender - this->descender;
+	if (err == FT_Err_Ok) {
+		this->units_per_em = this->face->units_per_EM;
+		this->ascender     = this->face->size->metrics.ascender >> 6;
+		this->descender    = this->face->size->metrics.descender >> 6;
+		this->height       = this->ascender - this->descender;
+	} else {
+		/* Both FT_Set_Pixel_Sizes and FT_Select_Size failed. */
+		DEBUG(freetype, 0, "Font size selection failed. Using FontCache defaults.");
+	}
 }
 
 /**
@@ -437,7 +451,7 @@ static void *AllocateFont(size_t size)
 static bool GetFontAAState(FontSize size)
 {
 	/* AA is only supported for 32 bpp */
-	if (BlitterFactoryBase::GetCurrentBlitter()->GetScreenDepth() != 32) return false;
+	if (BlitterFactory::GetCurrentBlitter()->GetScreenDepth() != 32) return false;
 
 	switch (size) {
 		default: NOT_REACHED();
@@ -492,7 +506,7 @@ const Sprite *FreeTypeFontCache::GetGlyph(GlyphID key)
 				builtin_questionmark_data
 			};
 
-			Sprite *spr = BlitterFactoryBase::GetCurrentBlitter()->Encode(&builtin_questionmark, AllocateFont);
+			Sprite *spr = BlitterFactory::GetCurrentBlitter()->Encode(&builtin_questionmark, AllocateFont);
 			assert(spr != NULL);
 			new_glyph.sprite = spr;
 			new_glyph.width  = spr->width + (this->fs != FS_NORMAL);
@@ -549,7 +563,7 @@ const Sprite *FreeTypeFontCache::GetGlyph(GlyphID key)
 		}
 	}
 
-	new_glyph.sprite = BlitterFactoryBase::GetCurrentBlitter()->Encode(&sprite, AllocateFont);
+	new_glyph.sprite = BlitterFactory::GetCurrentBlitter()->Encode(&sprite, AllocateFont);
 	new_glyph.width  = slot->advance.x >> 6;
 
 	this->SetGlyphPtr(key, &new_glyph);
