@@ -1,4 +1,4 @@
-/* $Id: gfx.cpp 26024 2013-11-17 13:35:48Z rubidium $ */
+/* $Id: gfx.cpp 26793 2014-09-07 15:41:03Z frosch $ */
 
 /*
  * This file is part of OpenTTD.
@@ -73,7 +73,7 @@ extern uint _dirty_block_colour;
 
 void GfxScroll(int left, int top, int width, int height, int xo, int yo)
 {
-	Blitter *blitter = BlitterFactoryBase::GetCurrentBlitter();
+	Blitter *blitter = BlitterFactory::GetCurrentBlitter();
 
 	if (xo == 0 && yo == 0) return;
 
@@ -85,7 +85,7 @@ void GfxScroll(int left, int top, int width, int height, int xo, int yo)
 
 	blitter->ScrollBuffer(_screen.dst_ptr, left, top, width, height, xo, yo);
 	/* This part of the screen is now dirty. */
-	_video_driver->MakeDirty(left, top, width, height);
+	VideoDriver::GetInstance()->MakeDirty(left, top, width, height);
 }
 
 
@@ -105,7 +105,7 @@ void GfxScroll(int left, int top, int width, int height, int xo, int yo)
  */
 void GfxFillRect(int left, int top, int right, int bottom, int colour, FillRectMode mode)
 {
-	Blitter *blitter = BlitterFactoryBase::GetCurrentBlitter();
+	Blitter *blitter = BlitterFactory::GetCurrentBlitter();
 	const DrawPixelInfo *dpi = _cur_dpi;
 	void *dst;
 	const int otop = top;
@@ -150,46 +150,115 @@ void GfxFillRect(int left, int top, int right, int bottom, int colour, FillRectM
 	}
 }
 
-void GfxDrawLine(int x, int y, int x2, int y2, int colour, int width)
+/**
+ * Check line clipping by using a linear equation and draw the visible part of
+ * the line given by x/y and x2/y2.
+ * @param video Destination pointer to draw into.
+ * @param x X coordinate of first point.
+ * @param y Y coordinate of first point.
+ * @param x2 X coordinate of second point.
+ * @param y2 Y coordinate of second point.
+ * @param screen_width With of the screen to check clipping against.
+ * @param screen_height Height of the screen to check clipping against.
+ * @param colour Colour of the line.
+ * @param width Width of the line.
+ * @param dash Length of dashes for dashed lines. 0 means solid line.
+ */
+static inline void GfxDoDrawLine(void *video, int x, int y, int x2, int y2, int screen_width, int screen_height, uint8 colour, int width, int dash = 0)
 {
-	Blitter *blitter = BlitterFactoryBase::GetCurrentBlitter();
-	DrawPixelInfo *dpi = _cur_dpi;
+	Blitter *blitter = BlitterFactory::GetCurrentBlitter();
 
 	assert(width > 0);
 
+	if (y2 == y) {
+		/* Special case: horizontal line. */
+		blitter->DrawLine(video,
+				Clamp(x, 0, screen_width), y,
+				Clamp(x2, 0, screen_width), y2,
+				screen_width, screen_height, colour, width, dash);
+		return;
+	}
+	if (x2 == x) {
+		/* Special case: vertical line. */
+		blitter->DrawLine(video,
+				x, Clamp(y, 0, screen_height),
+				x2, Clamp(y2, 0, screen_height),
+				screen_width, screen_height, colour, width, dash);
+		return;
+	}
+
+	int grade_y = y2 - y;
+	int grade_x = x2 - x;
+
+	/* prevent integer overflows. */
+	int margin = 1;
+	while (INT_MAX / abs(grade_y) < max(abs(x), abs(screen_width - x))) {
+		grade_y /= 2;
+		grade_x /= 2;
+		margin  *= 2; // account for rounding errors
+	}
+
+	/* If the line is outside the screen on the same side at X positions 0
+	 * and screen_width, we don't need to draw anything. */
+	int offset_0 = y - x * grade_y / grade_x;
+	int offset_width = y + (screen_width - x) * grade_y / grade_x;
+	if ((offset_0 > screen_height + width / 2 + margin && offset_width > screen_height + width / 2 + margin) ||
+			(offset_0 < -width / 2 - margin && offset_width < -width / 2 - margin)) {
+		return;
+	}
+
+	/* It is possible to use the line equation to further reduce the amount of
+	 * work the blitter has to do by shortening the effective line segment.
+	 * However, in order to get that right and prevent the flickering effects
+	 * of rounding errors so much additional code has to be run here that in
+	 * the general case the effect is not noticable. */
+
+	blitter->DrawLine(video, x, y, x2, y2, screen_width, screen_height, colour, width, dash);
+}
+
+/**
+ * Align parameters of a line to the given DPI and check simple clipping.
+ * @param dpi Screen parameters to align with.
+ * @param x X coordinate of first point.
+ * @param y Y coordinate of first point.
+ * @param x2 X coordinate of second point.
+ * @param y2 Y coordinate of second point.
+ * @param width Width of the line.
+ * @return True if the line is likely to be visible, false if it's certainly
+ *         invisible.
+ */
+static inline bool GfxPreprocessLine(DrawPixelInfo *dpi, int &x, int &y, int &x2, int &y2, int width)
+{
 	x -= dpi->left;
 	x2 -= dpi->left;
 	y -= dpi->top;
 	y2 -= dpi->top;
 
-	/* Check clipping */
-	if (x + width / 2 < 0           && x2 + width / 2 < 0          ) return;
-	if (y + width / 2 < 0           && y2 + width / 2 < 0          ) return;
-	if (x - width / 2 > dpi->width  && x2 - width / 2 > dpi->width ) return;
-	if (y - width / 2 > dpi->height && y2 - width / 2 > dpi->height) return;
+	/* Check simple clipping */
+	if (x + width / 2 < 0           && x2 + width / 2 < 0          ) return false;
+	if (y + width / 2 < 0           && y2 + width / 2 < 0          ) return false;
+	if (x - width / 2 > dpi->width  && x2 - width / 2 > dpi->width ) return false;
+	if (y - width / 2 > dpi->height && y2 - width / 2 > dpi->height) return false;
+	return true;
+}
 
-	blitter->DrawLine(dpi->dst_ptr, x, y, x2, y2, dpi->width, dpi->height, colour, width);
+void GfxDrawLine(int x, int y, int x2, int y2, int colour, int width, int dash)
+{
+	DrawPixelInfo *dpi = _cur_dpi;
+	if (GfxPreprocessLine(dpi, x, y, x2, y2, width)) {
+		GfxDoDrawLine(dpi->dst_ptr, x, y, x2, y2, dpi->width, dpi->height, colour, width, dash);
+	}
 }
 
 void GfxDrawLineUnscaled(int x, int y, int x2, int y2, int colour)
 {
-	Blitter *blitter = BlitterFactoryBase::GetCurrentBlitter();
 	DrawPixelInfo *dpi = _cur_dpi;
-
-	x -= dpi->left;
-	x2 -= dpi->left;
-	y -= dpi->top;
-	y2 -= dpi->top;
-
-	/* Check clipping */
-	if (x < 0 && x2 < 0) return;
-	if (y < 0 && y2 < 0) return;
-	if (x > dpi->width  && x2 > dpi->width)  return;
-	if (y > dpi->height && y2 > dpi->height) return;
-
-	blitter->DrawLine(dpi->dst_ptr, UnScaleByZoom(x, dpi->zoom), UnScaleByZoom(y, dpi->zoom),
-			UnScaleByZoom(x2, dpi->zoom), UnScaleByZoom(y2, dpi->zoom),
-			UnScaleByZoom(dpi->width, dpi->zoom), UnScaleByZoom(dpi->height, dpi->zoom), colour, 1);
+	if (GfxPreprocessLine(dpi, x, y, x2, y2, 1)) {
+		GfxDoDrawLine(dpi->dst_ptr,
+				UnScaleByZoom(x, dpi->zoom), UnScaleByZoom(y, dpi->zoom),
+				UnScaleByZoom(x2, dpi->zoom), UnScaleByZoom(y2, dpi->zoom),
+				UnScaleByZoom(dpi->width, dpi->zoom), UnScaleByZoom(dpi->height, dpi->zoom), colour, 1);
+	}
 }
 
 /**
@@ -349,19 +418,21 @@ static int DrawLayoutLine(const ParagraphLayouter::Line *line, int y, int left,
 			NOT_REACHED();
 	}
 
+	TextColour colour = TC_BLACK;
+	bool draw_shadow = false;
 	for (int run_index = 0; run_index < line->CountRuns(); run_index++) {
 		const ParagraphLayouter::VisualRun *run = line->GetVisualRun(run_index);
 		const Font *f = (const Font*)run->GetFont();
 
 		FontCache *fc = f->fc;
-		TextColour colour = f->colour;
+		colour = f->colour;
 		SetColourRemap(colour);
 
 		DrawPixelInfo *dpi = _cur_dpi;
 		int dpi_left  = dpi->left;
 		int dpi_right = dpi->left + dpi->width - 1;
 
-		bool draw_shadow = fc->GetDrawGlyphShadow() && colour != TC_BLACK;
+		draw_shadow = fc->GetDrawGlyphShadow() && (colour & TC_NO_SHADE) == 0 && colour != TC_BLACK;
 
 		for (int i = 0; i < run->GetGlyphCount(); i++) {
 			GlyphID glyph = run->GetGlyphs()[i];
@@ -392,6 +463,11 @@ static int DrawLayoutLine(const ParagraphLayouter::Line *line, int y, int left,
 	if (truncation) {
 		int x = (_current_text_dir == TD_RTL) ? left : (right - 3 * dot_width);
 		for (int i = 0; i < 3; i++, x += dot_width) {
+			if (draw_shadow) {
+				SetColourRemap(TC_BLACK);
+				GfxMainBlitter(dot_sprite, x + 1, y + 1, BM_COLOUR_REMAP);
+				SetColourRemap(colour);
+			}
 			GfxMainBlitter(dot_sprite, x, y, BM_COLOUR_REMAP);
 		}
 	}
@@ -467,9 +543,9 @@ int DrawString(int left, int right, int top, StringID str, TextColour colour, St
  * @param maxw maximum string width
  * @return height of pixels of string when it is drawn
  */
-static int GetStringHeight(const char *str, int maxw)
+int GetStringHeight(const char *str, int maxw, FontSize fontsize)
 {
-	Layouter layout(str, maxw);
+	Layouter layout(str, maxw, TC_FROMSTRING, fontsize);
 	return layout.GetBounds().height;
 }
 
@@ -719,7 +795,7 @@ void DrawSpriteViewport(SpriteID img, PaletteID pal, int x, int y, const SubSpri
 		GfxMainBlitterViewport(GetSprite(real_sprite, ST_NORMAL), x, y, BM_TRANSPARENT, sub, real_sprite);
 	} else if (pal != PAL_NONE) {
 		_colour_remap_ptr = GetNonSprite(GB(pal, 0, PALETTE_WIDTH), ST_RECOLOUR) + 1;
-		GfxMainBlitterViewport(GetSprite(real_sprite, ST_NORMAL), x, y, BM_COLOUR_REMAP, sub, real_sprite);
+		GfxMainBlitterViewport(GetSprite(real_sprite, ST_NORMAL), x, y, pal == PALETTE_CRASH ? BM_CRASH_REMAP : BM_COLOUR_REMAP, sub, real_sprite);
 	} else {
 		GfxMainBlitterViewport(GetSprite(real_sprite, ST_NORMAL), x, y, BM_NORMAL, sub, real_sprite);
 	}
@@ -742,43 +818,70 @@ void DrawSprite(SpriteID img, PaletteID pal, int x, int y, const SubSprite *sub,
 		GfxMainBlitter(GetSprite(real_sprite, ST_NORMAL), x, y, BM_TRANSPARENT, sub, real_sprite, zoom);
 	} else if (pal != PAL_NONE) {
 		_colour_remap_ptr = GetNonSprite(GB(pal, 0, PALETTE_WIDTH), ST_RECOLOUR) + 1;
-		GfxMainBlitter(GetSprite(real_sprite, ST_NORMAL), x, y, BM_COLOUR_REMAP, sub, real_sprite, zoom);
+		GfxMainBlitter(GetSprite(real_sprite, ST_NORMAL), x, y, pal == PALETTE_CRASH ? BM_CRASH_REMAP : BM_COLOUR_REMAP, sub, real_sprite, zoom);
 	} else {
 		GfxMainBlitter(GetSprite(real_sprite, ST_NORMAL), x, y, BM_NORMAL, sub, real_sprite, zoom);
 	}
 }
 
-static void GfxMainBlitterViewport(const Sprite *sprite, int x, int y, BlitterMode mode, const SubSprite *sub, SpriteID sprite_id)
+/**
+ * The code for setting up the blitter mode and sprite information before finally drawing the sprite.
+ * @param sprite The sprite to draw.
+ * @param x      The X location to draw.
+ * @param y      The Y location to draw.
+ * @param mode   The settings for the blitter to pass.
+ * @param sub    Whether to only draw a sub set of the sprite.
+ * @param zoom   The zoom level at which to draw the sprites.
+ * @tparam ZOOM_BASE The factor required to get the sub sprite information into the right size.
+ * @tparam SCALED_XY Whether the X and Y are scaled or unscaled.
+ */
+template <int ZOOM_BASE, bool SCALED_XY>
+static void GfxBlitter(const Sprite * const sprite, int x, int y, BlitterMode mode, const SubSprite * const sub, SpriteID sprite_id, ZoomLevel zoom)
 {
 	const DrawPixelInfo *dpi = _cur_dpi;
 	Blitter::BlitterParams bp;
 
-	/* Amount of pixels to clip from the source sprite */
-	int clip_left   = (sub != NULL ? max(0,                   -sprite->x_offs + sub->left * ZOOM_LVL_BASE       ) : 0);
-	int clip_top    = (sub != NULL ? max(0,                   -sprite->y_offs + sub->top  * ZOOM_LVL_BASE       ) : 0);
-	int clip_right  = (sub != NULL ? max(0, sprite->width  - (-sprite->x_offs + (sub->right + 1)  * ZOOM_LVL_BASE)) : 0);
-	int clip_bottom = (sub != NULL ? max(0, sprite->height - (-sprite->y_offs + (sub->bottom + 1) * ZOOM_LVL_BASE)) : 0);
-
-	if (clip_left + clip_right >= sprite->width) return;
-	if (clip_top + clip_bottom >= sprite->height) return;
+	if (SCALED_XY) {
+		/* Scale it */
+		x = ScaleByZoom(x, zoom);
+		y = ScaleByZoom(y, zoom);
+	}
 
 	/* Move to the correct offset */
 	x += sprite->x_offs;
 	y += sprite->y_offs;
 
+	if (sub == NULL) {
+		/* No clipping. */
+		bp.skip_left = 0;
+		bp.skip_top = 0;
+		bp.width = UnScaleByZoom(sprite->width, zoom);
+		bp.height = UnScaleByZoom(sprite->height, zoom);
+	} else {
+		/* Amount of pixels to clip from the source sprite */
+		int clip_left   = max(0,                   -sprite->x_offs +  sub->left        * ZOOM_BASE );
+		int clip_top    = max(0,                   -sprite->y_offs +  sub->top         * ZOOM_BASE );
+		int clip_right  = max(0, sprite->width  - (-sprite->x_offs + (sub->right + 1)  * ZOOM_BASE));
+		int clip_bottom = max(0, sprite->height - (-sprite->y_offs + (sub->bottom + 1) * ZOOM_BASE));
+
+		if (clip_left + clip_right >= sprite->width) return;
+		if (clip_top + clip_bottom >= sprite->height) return;
+
+		bp.skip_left = UnScaleByZoomLower(clip_left, zoom);
+		bp.skip_top = UnScaleByZoomLower(clip_top, zoom);
+		bp.width = UnScaleByZoom(sprite->width - clip_left - clip_right, zoom);
+		bp.height = UnScaleByZoom(sprite->height - clip_top - clip_bottom, zoom);
+
+		x += ScaleByZoom(bp.skip_left, zoom);
+		y += ScaleByZoom(bp.skip_top, zoom);
+	}
+
 	/* Copy the main data directly from the sprite */
 	bp.sprite = sprite->data;
 	bp.sprite_width = sprite->width;
 	bp.sprite_height = sprite->height;
-	bp.width = UnScaleByZoom(sprite->width - clip_left - clip_right, dpi->zoom);
-	bp.height = UnScaleByZoom(sprite->height - clip_top - clip_bottom, dpi->zoom);
 	bp.top = 0;
 	bp.left = 0;
-	bp.skip_left = UnScaleByZoomLower(clip_left, dpi->zoom);
-	bp.skip_top = UnScaleByZoomLower(clip_top, dpi->zoom);
-
-	x += ScaleByZoom(bp.skip_left, dpi->zoom);
-	y += ScaleByZoom(bp.skip_top, dpi->zoom);
 
 	bp.dst = dpi->dst_ptr;
 	bp.pitch = dpi->pitch;
@@ -790,141 +893,39 @@ static void GfxMainBlitterViewport(const Sprite *sprite, int x, int y, BlitterMo
 	if (bp.width <= 0) return;
 	if (bp.height <= 0) return;
 
-	y -= dpi->top;
+	y -= SCALED_XY ? ScaleByZoom(dpi->top, zoom) : dpi->top;
+	int y_unscaled = UnScaleByZoom(y, zoom);
 	/* Check for top overflow */
 	if (y < 0) {
-		bp.height -= -UnScaleByZoom(y, dpi->zoom);
+		bp.height -= -y_unscaled;
 		if (bp.height <= 0) return;
-		bp.skip_top += -UnScaleByZoom(y, dpi->zoom);
+		bp.skip_top += -y_unscaled;
 		y = 0;
 	} else {
-		bp.top = UnScaleByZoom(y, dpi->zoom);
+		bp.top = y_unscaled;
 	}
 
 	/* Check for bottom overflow */
-	y += ScaleByZoom(bp.height, dpi->zoom) - dpi->height;
-	if (y > 0) {
-		bp.height -= UnScaleByZoom(y, dpi->zoom);
-		if (bp.height <= 0) return;
-	}
-
-	x -= dpi->left;
-	/* Check for left overflow */
-	if (x < 0) {
-		bp.width -= -UnScaleByZoom(x, dpi->zoom);
-		if (bp.width <= 0) return;
-		bp.skip_left += -UnScaleByZoom(x, dpi->zoom);
-		x = 0;
-	} else {
-		bp.left = UnScaleByZoom(x, dpi->zoom);
-	}
-
-	/* Check for right overflow */
-	x += ScaleByZoom(bp.width, dpi->zoom) - dpi->width;
-	if (x > 0) {
-		bp.width -= UnScaleByZoom(x, dpi->zoom);
-		if (bp.width <= 0) return;
-	}
-
-	assert(bp.skip_left + bp.width <= UnScaleByZoom(sprite->width, dpi->zoom));
-	assert(bp.skip_top + bp.height <= UnScaleByZoom(sprite->height, dpi->zoom));
-
-	/* We do not want to catch the mouse. However we also use that spritenumber for unknown (text) sprites. */
-	if (_newgrf_debug_sprite_picker.mode == SPM_REDRAW && sprite_id != SPR_CURSOR_MOUSE) {
-		Blitter *blitter = BlitterFactoryBase::GetCurrentBlitter();
-		void *topleft = blitter->MoveTo(bp.dst, bp.left, bp.top);
-		void *bottomright = blitter->MoveTo(topleft, bp.width - 1, bp.height - 1);
-
-		void *clicked = _newgrf_debug_sprite_picker.clicked_pixel;
-
-		if (topleft <= clicked && clicked <= bottomright) {
-			uint offset = (((size_t)clicked - (size_t)topleft) / (blitter->GetScreenDepth() / 8)) % bp.pitch;
-			if (offset < (uint)bp.width) {
-				_newgrf_debug_sprite_picker.sprites.Include(sprite_id);
-			}
-		}
-	}
-
-	BlitterFactoryBase::GetCurrentBlitter()->Draw(&bp, mode, dpi->zoom);
-}
-
-static void GfxMainBlitter(const Sprite *sprite, int x, int y, BlitterMode mode, const SubSprite *sub, SpriteID sprite_id, ZoomLevel zoom)
-{
-	const DrawPixelInfo *dpi = _cur_dpi;
-	Blitter::BlitterParams bp;
-
-	/* Amount of pixels to clip from the source sprite */
-	int clip_left   = (sub != NULL ? max(0,                   -sprite->x_offs + sub->left       ) : 0);
-	int clip_top    = (sub != NULL ? max(0,                   -sprite->y_offs + sub->top        ) : 0);
-	int clip_right  = (sub != NULL ? max(0, sprite->width  - (-sprite->x_offs + sub->right  + 1)) : 0);
-	int clip_bottom = (sub != NULL ? max(0, sprite->height - (-sprite->y_offs + sub->bottom + 1)) : 0);
-
-	if (clip_left + clip_right >= sprite->width) return;
-	if (clip_top + clip_bottom >= sprite->height) return;
-
-	/* Scale it */
-	x = ScaleByZoom(x, zoom);
-	y = ScaleByZoom(y, zoom);
-
-	/* Move to the correct offset */
-	x += sprite->x_offs;
-	y += sprite->y_offs;
-
-	/* Copy the main data directly from the sprite */
-	bp.sprite = sprite->data;
-	bp.sprite_width = sprite->width;
-	bp.sprite_height = sprite->height;
-	bp.width = UnScaleByZoom(sprite->width - clip_left - clip_right, zoom);
-	bp.height = UnScaleByZoom(sprite->height - clip_top - clip_bottom, zoom);
-	bp.top = 0;
-	bp.left = 0;
-	bp.skip_left = UnScaleByZoomLower(clip_left, zoom);
-	bp.skip_top = UnScaleByZoomLower(clip_top, zoom);
-
-	x += ScaleByZoom(bp.skip_left, zoom);
-	y += ScaleByZoom(bp.skip_top, zoom);
-
-	bp.dst = dpi->dst_ptr;
-	bp.pitch = dpi->pitch;
-	bp.remap = _colour_remap_ptr;
-
-	assert(sprite->width > 0);
-	assert(sprite->height > 0);
-
-	if (bp.width <= 0) return;
-	if (bp.height <= 0) return;
-
-	y -= ScaleByZoom(dpi->top, zoom);
-	/* Check for top overflow */
-	if (y < 0) {
-		bp.height -= -UnScaleByZoom(y, zoom);
-		if (bp.height <= 0) return;
-		bp.skip_top += -UnScaleByZoom(y, zoom);
-		y = 0;
-	} else {
-		bp.top = UnScaleByZoom(y, zoom);
-	}
-
-	/* Check for bottom overflow */
-	y += ScaleByZoom(bp.height - dpi->height, zoom);
+	y += SCALED_XY ? ScaleByZoom(bp.height - dpi->height, zoom) : ScaleByZoom(bp.height, zoom) - dpi->height;
 	if (y > 0) {
 		bp.height -= UnScaleByZoom(y, zoom);
 		if (bp.height <= 0) return;
 	}
 
-	x -= ScaleByZoom(dpi->left, zoom);
+	x -= SCALED_XY ? ScaleByZoom(dpi->left, zoom) : dpi->left;
+	int x_unscaled = UnScaleByZoom(x, zoom);
 	/* Check for left overflow */
 	if (x < 0) {
-		bp.width -= -UnScaleByZoom(x, zoom);
+		bp.width -= -x_unscaled;
 		if (bp.width <= 0) return;
-		bp.skip_left += -UnScaleByZoom(x, zoom);
+		bp.skip_left += -x_unscaled;
 		x = 0;
 	} else {
-		bp.left = UnScaleByZoom(x, zoom);
+		bp.left = x_unscaled;
 	}
 
 	/* Check for right overflow */
-	x += ScaleByZoom(bp.width - dpi->width, zoom);
+	x += SCALED_XY ? ScaleByZoom(bp.width - dpi->width, zoom) : ScaleByZoom(bp.width, zoom) - dpi->width;
 	if (x > 0) {
 		bp.width -= UnScaleByZoom(x, zoom);
 		if (bp.width <= 0) return;
@@ -935,7 +936,7 @@ static void GfxMainBlitter(const Sprite *sprite, int x, int y, BlitterMode mode,
 
 	/* We do not want to catch the mouse. However we also use that spritenumber for unknown (text) sprites. */
 	if (_newgrf_debug_sprite_picker.mode == SPM_REDRAW && sprite_id != SPR_CURSOR_MOUSE) {
-		Blitter *blitter = BlitterFactoryBase::GetCurrentBlitter();
+		Blitter *blitter = BlitterFactory::GetCurrentBlitter();
 		void *topleft = blitter->MoveTo(bp.dst, bp.left, bp.top);
 		void *bottomright = blitter->MoveTo(topleft, bp.width - 1, bp.height - 1);
 
@@ -949,7 +950,17 @@ static void GfxMainBlitter(const Sprite *sprite, int x, int y, BlitterMode mode,
 		}
 	}
 
-	BlitterFactoryBase::GetCurrentBlitter()->Draw(&bp, mode, zoom);
+	BlitterFactory::GetCurrentBlitter()->Draw(&bp, mode, zoom);
+}
+
+static void GfxMainBlitterViewport(const Sprite *sprite, int x, int y, BlitterMode mode, const SubSprite *sub, SpriteID sprite_id)
+{
+	GfxBlitter<ZOOM_LVL_BASE, false>(sprite, x, y, mode, sub, sprite_id, _cur_dpi->zoom);
+}
+
+static void GfxMainBlitter(const Sprite *sprite, int x, int y, BlitterMode mode, const SubSprite *sub, SpriteID sprite_id, ZoomLevel zoom)
+{
+	GfxBlitter<1, true>(sprite, x, y, mode, sub, sprite_id, zoom);
 }
 
 void DoPaletteAnimations();
@@ -969,7 +980,7 @@ void DoPaletteAnimations()
 	static int palette_animation_counter = 0;
 	palette_animation_counter += 8;
 
-	Blitter *blitter = BlitterFactoryBase::GetCurrentBlitter();
+	Blitter *blitter = BlitterFactory::GetCurrentBlitter();
 	const Colour *s;
 	const ExtraPaletteValues *ev = &_extra_palette_values;
 	Colour old_val[PALETTE_ANIM_SIZE];
@@ -1169,10 +1180,10 @@ void UndrawMouseCursor()
 	if (_screen.dst_ptr == NULL) return;
 
 	if (_cursor.visible) {
-		Blitter *blitter = BlitterFactoryBase::GetCurrentBlitter();
+		Blitter *blitter = BlitterFactory::GetCurrentBlitter();
 		_cursor.visible = false;
 		blitter->CopyFromBuffer(blitter->MoveTo(_screen.dst_ptr, _cursor.draw_pos.x, _cursor.draw_pos.y), _cursor_backup.GetBuffer(), _cursor.draw_size.x, _cursor.draw_size.y);
-		_video_driver->MakeDirty(_cursor.draw_pos.x, _cursor.draw_pos.y, _cursor.draw_size.x, _cursor.draw_size.y);
+		VideoDriver::GetInstance()->MakeDirty(_cursor.draw_pos.x, _cursor.draw_pos.y, _cursor.draw_size.x, _cursor.draw_size.y);
 	}
 }
 
@@ -1186,7 +1197,7 @@ void DrawMouseCursor()
 	/* Don't draw the mouse cursor if the screen is not ready */
 	if (_screen.dst_ptr == NULL) return;
 
-	Blitter *blitter = BlitterFactoryBase::GetCurrentBlitter();
+	Blitter *blitter = BlitterFactory::GetCurrentBlitter();
 	int x;
 	int y;
 	int w;
@@ -1232,7 +1243,7 @@ void DrawMouseCursor()
 	_cur_dpi = &_screen;
 	DrawSprite(_cursor.sprite, _cursor.pal, _cursor.pos.x + _cursor.short_vehicle_offset, _cursor.pos.y);
 
-	_video_driver->MakeDirty(_cursor.draw_pos.x, _cursor.draw_pos.y, _cursor.draw_size.x, _cursor.draw_size.y);
+	VideoDriver::GetInstance()->MakeDirty(_cursor.draw_pos.x, _cursor.draw_pos.y, _cursor.draw_size.x, _cursor.draw_size.y);
 
 	_cursor.visible = true;
 	_cursor.dirty = false;
@@ -1256,7 +1267,7 @@ void RedrawScreenRect(int left, int top, int right, int bottom)
 
 	DrawOverlappedWindowForAll(left, top, right, bottom);
 
-	_video_driver->MakeDirty(left, top, right - left, bottom - top);
+	VideoDriver::GetInstance()->MakeDirty(left, top, right - left, bottom - top);
 }
 
 /**
@@ -1439,7 +1450,7 @@ void MarkWholeScreenDirty()
  */
 bool FillDrawPixelInfo(DrawPixelInfo *n, int left, int top, int width, int height)
 {
-	Blitter *blitter = BlitterFactoryBase::GetCurrentBlitter();
+	Blitter *blitter = BlitterFactory::GetCurrentBlitter();
 	const DrawPixelInfo *o = _cur_dpi;
 
 	n->zoom = ZOOM_LVL_NORMAL;
@@ -1565,12 +1576,12 @@ void SetAnimatedMouseCursor(const AnimCursor *table)
 
 bool ChangeResInGame(int width, int height)
 {
-	return (_screen.width == width && _screen.height == height) || _video_driver->ChangeResolution(width, height);
+	return (_screen.width == width && _screen.height == height) || VideoDriver::GetInstance()->ChangeResolution(width, height);
 }
 
 bool ToggleFullScreen(bool fs)
 {
-	bool result = _video_driver->ToggleFullscreen(fs);
+	bool result = VideoDriver::GetInstance()->ToggleFullscreen(fs);
 	if (_fullscreen != fs && _num_resolutions == 0) {
 		DEBUG(driver, 0, "Could not find a suitable fullscreen resolution");
 	}
