@@ -1,4 +1,4 @@
-/* $Id: gui.h 24791 2012-12-05 19:37:15Z frosch $ */
+/* $Id: gui.h 25372 2013-06-09 13:23:03Z zuu $ */
 
 /*
  * This file is part of OpenTTD.
@@ -16,6 +16,8 @@
 #include "economy_type.h"
 #include "tile_type.h"
 #include "transport_type.h"
+#include "story_type.h"
+#include "company_type.h"
 
 struct Window;
 
@@ -49,8 +51,9 @@ void ShowTownDirectory();
 void ShowIndustryDirectory();
 void ShowIndustryCargoesWindow();
 void ShowSubsidiesList();
-void ShowGoalsList();
+void ShowGoalsList(CompanyID company);
 void ShowGoalQuestion(uint16 id, byte type, uint32 button_mask, const char *question);
+void ShowStoryBook(CompanyID company, uint16 page_id = INVALID_STORY_PAGE);
 
 void ShowEstimatedCostOrIncome(Money cost, int x, int y);
 
