@@ -1,4 +1,4 @@
-/* $Id: network.cpp 24900 2013-01-08 22:46:42Z planetmaker $ */
+/* $Id: network.cpp 26595 2014-05-18 11:21:59Z frosch $ */
 
 /*
  * This file is part of OpenTTD.
@@ -698,7 +698,7 @@ static void NetworkInitGameInfo()
 	/* There should be always space for the server. */
 	assert(NetworkClientInfo::CanAllocateItem());
 	NetworkClientInfo *ci = new NetworkClientInfo(CLIENT_ID_SERVER);
-	ci->client_playas = _network_dedicated ? COMPANY_SPECTATOR : _local_company;
+	ci->client_playas = _network_dedicated ? COMPANY_SPECTATOR : COMPANY_FIRST;
 
 	strecpy(ci->client_name, _settings_client.network.client_name, lastof(ci->client_name));
 }
@@ -713,12 +713,17 @@ bool NetworkServerStart()
 
 	NetworkDisconnect(false, false);
 	NetworkInitialize(false);
+	DEBUG(net, 1, "starting listeners for clients");
 	if (!ServerNetworkGameSocketHandler::Listen(_settings_client.network.server_port)) return false;
 
 	/* Only listen for admins when the password isn't empty. */
-	if (!StrEmpty(_settings_client.network.admin_password) && !ServerNetworkAdminSocketHandler::Listen(_settings_client.network.server_admin_port)) return false;
+	if (!StrEmpty(_settings_client.network.admin_password)) {
+		DEBUG(net, 1, "starting listeners for admins");
+		if (!ServerNetworkAdminSocketHandler::Listen(_settings_client.network.server_admin_port)) return false;
+	}
 
 	/* Try to start UDP-server */
+	DEBUG(net, 1, "starting listeners for incoming server queries");
 	_network_udp_server = _udp_server_socket->Listen();
 
 	_network_company_states = CallocT<NetworkCompanyState>(MAX_COMPANIES);
