@@ -1,4 +1,4 @@
-/* $Id: newgrf_config.cpp 24318 2012-06-01 15:20:18Z frosch $ */
+/* $Id: newgrf_config.cpp 26544 2014-04-29 18:41:19Z frosch $ */
 
 /*
  * This file is part of OpenTTD.
@@ -226,7 +226,8 @@ GRFParameterInfo::GRFParameterInfo(uint nr) :
 	def_value(0),
 	param_nr(nr),
 	first_bit(0),
-	num_bit(32)
+	num_bit(32),
+	complete_labels(false)
 {}
 
 /**
@@ -334,7 +335,14 @@ size_t GRFGetSizeOfDataSection(FILE *f)
 	if (fread(data, 1, header_len, f) == header_len) {
 		if (data[0] == 0 && data[1] == 0 && MemCmpT(data + 2, _grf_cont_v2_sig, 8) == 0) {
 			/* Valid container version 2, get data section size. */
-			size_t offset = (data[13] << 24) | (data[12] << 16) | (data[11] << 8) | data[10];
+			size_t offset = ((size_t)data[13] << 24) | ((size_t)data[12] << 16) | ((size_t)data[11] << 8) | (size_t)data[10];
+			if (offset >= 1 * 1024 * 1024 * 1024) {
+				DEBUG(grf, 0, "Unexpectedly large offset for NewGRF");
+				/* Having more than 1 GiB of data is very implausible. Mostly because then
+				 * all pools in OpenTTD are flooded already. Or it's just Action C all over.
+				 * In any case, the offsets to graphics will likely not work either. */
+				return SIZE_MAX;
+			}
 			return header_len + offset;
 		}
 	}
@@ -359,9 +367,13 @@ static bool CalcGRFMD5Sum(GRFConfig *config, Subdirectory subdir)
 	f = FioFOpenFile(config->filename, "rb", subdir, &size);
 	if (f == NULL) return false;
 
-	size_t start = ftell(f);
+	long start = ftell(f);
 	size = min(size, GRFGetSizeOfDataSection(f));
-	fseek(f, start, SEEK_SET);
+
+	if (start < 0 || fseek(f, start, SEEK_SET) < 0) {
+		FioFCloseFile(f);
+		return false;
+	}
 
 	/* calculate md5sum */
 	while ((len = fread(buffer, 1, (size > sizeof(buffer)) ? sizeof(buffer) : size, f)) != 0 && size != 0) {
@@ -761,7 +773,7 @@ void ScanNewGRFFiles(NewGRFScanCallback *callback)
 	/* Only then can we really start, especially by marking the whole screen dirty. Get those other windows hidden!. */
 	MarkWholeScreenDirty();
 
-	if (!_video_driver->HasGUI() || !ThreadObject::New(&DoScanNewGRFFiles, callback, NULL)) {
+	if (!VideoDriver::GetInstance()->HasGUI() || !ThreadObject::New(&DoScanNewGRFFiles, callback, NULL)) {
 		_modal_progress_work_mutex->EndCritical();
 		_modal_progress_paint_mutex->EndCritical();
 		DoScanNewGRFFiles(callback);
