@@ -1,4 +1,4 @@
-/* $Id: newgrf_spritegroup.cpp 24900 2013-01-08 22:46:42Z planetmaker $ */
+/* $Id: newgrf_spritegroup.cpp 26541 2014-04-29 18:18:52Z frosch $ */
 
 /*
  * This file is part of OpenTTD.
@@ -17,6 +17,28 @@
 SpriteGroupPool _spritegroup_pool("SpriteGroup");
 INSTANTIATE_POOL_METHODS(SpriteGroup)
 
+TemporaryStorageArray<int32, 0x110> _temp_store;
+
+
+/**
+ * ResolverObject (re)entry point.
+ * This cannot be made a call to a virtual function because virtual functions
+ * do not like NULL and checking for NULL *everywhere* is more cumbersome than
+ * this little helper function.
+ * @param group the group to resolve for
+ * @param object information needed to resolve the group
+ * @param top_level true if this is a top-level SpriteGroup, false if used nested in another SpriteGroup.
+ * @return the resolved group
+ */
+/* static */ const SpriteGroup *SpriteGroup::Resolve(const SpriteGroup *group, ResolverObject &object, bool top_level)
+{
+	if (group == NULL) return NULL;
+	if (top_level) {
+		_temp_store.ClearChanges();
+	}
+	return group->Resolve(object);
+}
+
 RealSpriteGroup::~RealSpriteGroup()
 {
 	free(this->loaded);
@@ -34,38 +56,35 @@ RandomizedSpriteGroup::~RandomizedSpriteGroup()
 	free(this->groups);
 }
 
-TemporaryStorageArray<int32, 0x110> _temp_store;
-
-
-static inline uint32 GetVariable(const ResolverObject *object, ScopeResolver *scope, byte variable, uint32 parameter, bool *available)
+static inline uint32 GetVariable(const ResolverObject &object, ScopeResolver *scope, byte variable, uint32 parameter, bool *available)
 {
 	/* First handle variables common with Action7/9/D */
 	uint32 value;
-	if (GetGlobalVariable(variable, &value, object->grffile)) return value;
+	if (GetGlobalVariable(variable, &value, object.grffile)) return value;
 
 	/* Non-common variable */
 	switch (variable) {
-		case 0x0C: return object->callback;
-		case 0x10: return object->callback_param1;
-		case 0x18: return object->callback_param2;
-		case 0x1C: return object->last_value;
+		case 0x0C: return object.callback;
+		case 0x10: return object.callback_param1;
+		case 0x18: return object.callback_param2;
+		case 0x1C: return object.last_value;
 
 		case 0x5F: return (scope->GetRandomBits() << 8) | scope->GetTriggers();
 
 		case 0x7D: return _temp_store.GetValue(parameter);
 
 		case 0x7F:
-			if (object == NULL || object->grffile == NULL) return 0;
-			return object->grffile->GetParam(parameter);
+			if (object.grffile == NULL) return 0;
+			return object.grffile->GetParam(parameter);
 
 		/* Not a common variable, so evaluate the feature specific variables */
 		default: return scope->GetVariable(variable, parameter, available);
 	}
 }
 
-ScopeResolver::ScopeResolver(ResolverObject *ro)
+ScopeResolver::ScopeResolver(ResolverObject &ro)
+		: ro(ro)
 {
-	this->ro = ro;
 }
 
 ScopeResolver::~ScopeResolver() {}
@@ -123,7 +142,7 @@ ScopeResolver::~ScopeResolver() {}
  * @param callback_param2 Second parameter (var 18) of the callback (only used when \a callback is also set).
  */
 ResolverObject::ResolverObject(const GRFFile *grffile, CallbackID callback, uint32 callback_param1, uint32 callback_param2)
-		: default_scope(this)
+		: default_scope(*this)
 {
 	this->callback = callback;
 	this->callback_param1 = callback_param1;
@@ -131,6 +150,7 @@ ResolverObject::ResolverObject(const GRFFile *grffile, CallbackID callback, uint
 	this->ResetState();
 
 	this->grffile = grffile;
+	this->root_spritegroup = NULL;
 }
 
 ResolverObject::~ResolverObject() {}
@@ -208,21 +228,21 @@ static U EvalAdjustT(const DeterministicSpriteGroupAdjust *adjust, ScopeResolver
 		case DSGA_OP_ROR:  return RotateRight(last_value, value);
 		case DSGA_OP_SCMP: return ((S)last_value == (S)value) ? 1 : ((S)last_value < (S)value ? 0 : 2);
 		case DSGA_OP_UCMP: return ((U)last_value == (U)value) ? 1 : ((U)last_value < (U)value ? 0 : 2);
-		case DSGA_OP_SHL:  return (U)last_value << ((U)value & 0x1F); // mask 'value' to 5 bits, which should behave the same on all architectures.
-		case DSGA_OP_SHR:  return (U)last_value >> ((U)value & 0x1F);
-		case DSGA_OP_SAR:  return (S)last_value >> ((U)value & 0x1F);
+		case DSGA_OP_SHL:  return (uint32)(U)last_value << ((U)value & 0x1F); // Same behaviour as in ParamSet, mask 'value' to 5 bits, which should behave the same on all architectures.
+		case DSGA_OP_SHR:  return (uint32)(U)last_value >> ((U)value & 0x1F);
+		case DSGA_OP_SAR:  return (int32)(S)last_value >> ((U)value & 0x1F);
 		default:           return value;
 	}
 }
 
 
-const SpriteGroup *DeterministicSpriteGroup::Resolve(ResolverObject *object) const
+const SpriteGroup *DeterministicSpriteGroup::Resolve(ResolverObject &object) const
 {
 	uint32 last_value = 0;
 	uint32 value = 0;
 	uint i;
 
-	ScopeResolver *scope = object->GetScope(this->var_scope);
+	ScopeResolver *scope = object.GetScope(this->var_scope);
 
 	for (i = 0; i < this->num_adjusts; i++) {
 		DeterministicSpriteGroupAdjust *adjust = &this->adjusts[i];
@@ -230,7 +250,7 @@ const SpriteGroup *DeterministicSpriteGroup::Resolve(ResolverObject *object) con
 		/* Try to get the variable. We shall assume it is available, unless told otherwise. */
 		bool available = true;
 		if (adjust->variable == 0x7E) {
-			const SpriteGroup *subgroup = SpriteGroup::Resolve(adjust->subroutine, object);
+			const SpriteGroup *subgroup = SpriteGroup::Resolve(adjust->subroutine, object, false);
 			if (subgroup == NULL) {
 				value = CALLBACK_FAILED;
 			} else {
@@ -247,7 +267,7 @@ const SpriteGroup *DeterministicSpriteGroup::Resolve(ResolverObject *object) con
 		if (!available) {
 			/* Unsupported variable: skip further processing and return either
 			 * the group from the first range or the default group. */
-			return SpriteGroup::Resolve(this->num_ranges > 0 ? this->ranges[0].group : this->default_group, object);
+			return SpriteGroup::Resolve(this->num_ranges > 0 ? this->ranges[0].group : this->default_group, object, false);
 		}
 
 		switch (this->size) {
@@ -259,7 +279,7 @@ const SpriteGroup *DeterministicSpriteGroup::Resolve(ResolverObject *object) con
 		last_value = value;
 	}
 
-	object->last_value = last_value;
+	object.last_value = last_value;
 
 	if (this->num_ranges == 0) {
 		/* nvar == 0 is a special case -- we turn our value into a callback result */
@@ -271,29 +291,29 @@ const SpriteGroup *DeterministicSpriteGroup::Resolve(ResolverObject *object) con
 
 	for (i = 0; i < this->num_ranges; i++) {
 		if (this->ranges[i].low <= value && value <= this->ranges[i].high) {
-			return SpriteGroup::Resolve(this->ranges[i].group, object);
+			return SpriteGroup::Resolve(this->ranges[i].group, object, false);
 		}
 	}
 
-	return SpriteGroup::Resolve(this->default_group, object);
+	return SpriteGroup::Resolve(this->default_group, object, false);
 }
 
 
-const SpriteGroup *RandomizedSpriteGroup::Resolve(ResolverObject *object) const
+const SpriteGroup *RandomizedSpriteGroup::Resolve(ResolverObject &object) const
 {
-	ScopeResolver *scope = object->GetScope(this->var_scope, this->count);
-	if (object->trigger != 0) {
+	ScopeResolver *scope = object.GetScope(this->var_scope, this->count);
+	if (object.trigger != 0) {
 		/* Handle triggers */
 		/* Magic code that may or may not do the right things... */
 		byte waiting_triggers = scope->GetTriggers();
-		byte match = this->triggers & (waiting_triggers | object->trigger);
+		byte match = this->triggers & (waiting_triggers | object.trigger);
 		bool res = (this->cmp_mode == RSG_CMP_ANY) ? (match != 0) : (match == this->triggers);
 
 		if (res) {
 			waiting_triggers &= ~match;
-			object->reseed[this->var_scope] |= (this->num_groups - 1) << this->lowest_randbit;
+			object.reseed[this->var_scope] |= (this->num_groups - 1) << this->lowest_randbit;
 		} else {
-			waiting_triggers |= object->trigger;
+			waiting_triggers |= object.trigger;
 		}
 
 		scope->SetTriggers(waiting_triggers);
@@ -302,13 +322,13 @@ const SpriteGroup *RandomizedSpriteGroup::Resolve(ResolverObject *object) const
 	uint32 mask  = (this->num_groups - 1) << this->lowest_randbit;
 	byte index = (scope->GetRandomBits() & mask) >> this->lowest_randbit;
 
-	return SpriteGroup::Resolve(this->groups[index], object);
+	return SpriteGroup::Resolve(this->groups[index], object, false);
 }
 
 
-const SpriteGroup *RealSpriteGroup::Resolve(ResolverObject *object) const
+const SpriteGroup *RealSpriteGroup::Resolve(ResolverObject &object) const
 {
-	return object->ResolveReal(this);
+	return object.ResolveReal(this);
 }
 
 /**
