@@ -1,4 +1,4 @@
-/* $Id: newgrf_text.cpp 22995 2011-10-04 20:22:46Z rubidium $ */
+/* $Id: newgrf_text.cpp 26715 2014-08-03 14:06:04Z frosch $ */
 
 /*
  * This file is part of OpenTTD.
@@ -22,6 +22,7 @@
 #include "strings_func.h"
 #include "newgrf_storage.h"
 #include "newgrf_text.h"
+#include "newgrf_cargo.h"
 #include "string_func.h"
 #include "date_type.h"
 #include "debug.h"
@@ -35,66 +36,6 @@
 #define GRFTAB  28
 #define TABSIZE 11
 
-/**
- * Perform a mapping from TTDPatch's string IDs to OpenTTD's
- * string IDs, but only for the ones we are aware off; the rest
- * like likely unused and will show a warning.
- * @param str the string ID to convert
- * @return the converted string ID
- */
-StringID TTDPStringIDToOTTDStringIDMapping(StringID str)
-{
-	/* StringID table for TextIDs 0x4E->0x6D */
-	static const StringID units_volume[] = {
-		STR_ITEMS,      STR_PASSENGERS, STR_TONS,       STR_BAGS,
-		STR_LITERS,     STR_ITEMS,      STR_CRATES,     STR_TONS,
-		STR_TONS,       STR_TONS,       STR_TONS,       STR_BAGS,
-		STR_TONS,       STR_TONS,       STR_TONS,       STR_BAGS,
-		STR_TONS,       STR_TONS,       STR_BAGS,       STR_LITERS,
-		STR_TONS,       STR_LITERS,     STR_TONS,       STR_ITEMS,
-		STR_BAGS,       STR_LITERS,     STR_TONS,       STR_ITEMS,
-		STR_TONS,       STR_ITEMS,      STR_LITERS,     STR_ITEMS
-	};
-
-	/* A string straight from a NewGRF; no need to remap this as it's already mapped. */
-	if (IsInsideMM(str, 0xD000, 0xD7FF)) return str;
-
-#define TEXTID_TO_STRINGID(begin, end, stringid) if (str >= begin && str <= end) return str + (stringid - begin)
-	/* We have some changes in our cargo strings, resulting in some missing. */
-	TEXTID_TO_STRINGID(0x000E, 0x002D, STR_CARGO_PLURAL_NOTHING);
-	TEXTID_TO_STRINGID(0x002E, 0x004D, STR_CARGO_SINGULAR_NOTHING);
-	if (str >= 0x004E && str <= 0x006D) return units_volume[str - 0x004E];
-	TEXTID_TO_STRINGID(0x006E, 0x008D, STR_QUANTITY_NOTHING);
-	TEXTID_TO_STRINGID(0x008E, 0x00AD, STR_ABBREV_NOTHING);
-
-	/* Map building names according to our lang file changes. There are several
-	 * ranges of house ids, all of which need to be remapped to allow newgrfs
-	 * to use original house names. */
-	TEXTID_TO_STRINGID(0x200F, 0x201F, STR_TOWN_BUILDING_NAME_TALL_OFFICE_BLOCK_1);
-	TEXTID_TO_STRINGID(0x2036, 0x2041, STR_TOWN_BUILDING_NAME_COTTAGES_1);
-	TEXTID_TO_STRINGID(0x2059, 0x205C, STR_TOWN_BUILDING_NAME_IGLOO_1);
-
-	/* Same thing for industries */
-	TEXTID_TO_STRINGID(0x4802, 0x4826, STR_INDUSTRY_NAME_COAL_MINE);
-	TEXTID_TO_STRINGID(0x482D, 0x482E, STR_NEWS_INDUSTRY_CONSTRUCTION);
-	TEXTID_TO_STRINGID(0x4832, 0x4834, STR_NEWS_INDUSTRY_CLOSURE_GENERAL);
-	TEXTID_TO_STRINGID(0x4835, 0x4838, STR_NEWS_INDUSTRY_PRODUCTION_INCREASE_GENERAL);
-	TEXTID_TO_STRINGID(0x4839, 0x483A, STR_NEWS_INDUSTRY_PRODUCTION_DECREASE_GENERAL);
-
-	switch (str) {
-		case 0x4830: return STR_ERROR_CAN_T_CONSTRUCT_THIS_INDUSTRY;
-		case 0x4831: return STR_ERROR_FOREST_CAN_ONLY_BE_PLANTED;
-		case 0x483B: return STR_ERROR_CAN_ONLY_BE_POSITIONED;
-	}
-#undef TEXTID_TO_STRINGID
-
-	if (str == STR_NULL) return STR_EMPTY;
-
-	DEBUG(grf, 0, "Unknown StringID 0x%04X remapped to STR_EMPTY. Please open a Feature Request if you need it", str);
-
-	return STR_EMPTY;
-}
-
 /**
  * Explains the newgrf shift bit positioning.
  * the grf base will not be used in order to find the string, but rather for
@@ -178,7 +119,7 @@ private:
 	GRFText(byte langid_, const char *text_, size_t len_) : next(NULL), len(len_), langid(langid_)
 	{
 		/* We need to use memcpy instead of strcpy due to
-		 * the possibility of "choice lists" and therefor
+		 * the possibility of "choice lists" and therefore
 		 * intermediate string terminators. */
 		memcpy(this->text, text_, len);
 	}
@@ -292,7 +233,7 @@ struct UnmappedChoiceList : ZeroedMemoryAllocator {
 		}
 
 		char *d = old_d;
-		if (lm == NULL && this->type != SCC_PLURAL_LIST) {
+		if (lm == NULL) {
 			/* In case there is no mapping, just ignore everything but the default.
 			 * A probable cause for this happening is when the language file has
 			 * been removed by the user and as such no mapping could be made. */
@@ -373,7 +314,7 @@ struct UnmappedChoiceList : ZeroedMemoryAllocator {
 				const char *str = this->strings[this->strings.Contains(idx) ? idx : 0];
 				/* Limit the length of the string we copy to 0xFE. The length is written above
 				 * as a byte and we need room for the final '\0'. */
-				size_t len = min(0xFE, strlen(str));
+				size_t len = min<size_t>(0xFE, strlen(str));
 				memcpy(d, str, len);
 				d += len;
 				*d++ = '\0';
@@ -390,9 +331,10 @@ struct UnmappedChoiceList : ZeroedMemoryAllocator {
  * @param allow_newlines Whether newlines are allowed in the string or not.
  * @param str            The string to translate.
  * @param [out] olen     The length of the final string.
+ * @param byte80         The control code to use as replacement for the 0x80-value.
  * @return The translated string.
  */
-char *TranslateTTDPatchCodes(uint32 grfid, uint8 language_id, bool allow_newlines, const char *str, int *olen)
+char *TranslateTTDPatchCodes(uint32 grfid, uint8 language_id, bool allow_newlines, const char *str, int *olen, StringControlCode byte80)
 {
 	char *tmp = MallocT<char>(strlen(str) * 10 + 1); // Allocate space to allow for expansion
 	char *d = tmp;
@@ -427,8 +369,8 @@ char *TranslateTTDPatchCodes(uint32 grfid, uint8 language_id, bool allow_newline
 		switch (c) {
 			case 0x01:
 				if (str[0] == '\0') goto string_end;
-				d += Utf8Encode(d, SCC_SETX);
-				*d++ = *str++;
+				d += Utf8Encode(d, ' ');
+				str++;
 				break;
 			case 0x0A: break;
 			case 0x0D:
@@ -442,16 +384,15 @@ char *TranslateTTDPatchCodes(uint32 grfid, uint8 language_id, bool allow_newline
 			case 0x0F: d += Utf8Encode(d, SCC_BIGFONT); break;
 			case 0x1F:
 				if (str[0] == '\0' || str[1] == '\0') goto string_end;
-				d += Utf8Encode(d, SCC_SETXY);
-				*d++ = *str++;
-				*d++ = *str++;
+				d += Utf8Encode(d, ' ');
+				str += 2;
 				break;
 			case 0x7B:
 			case 0x7C:
 			case 0x7D:
 			case 0x7E:
-			case 0x7F:
-			case 0x80: d += Utf8Encode(d, SCC_NEWGRF_PRINT_DWORD + c - 0x7B); break;
+			case 0x7F: d += Utf8Encode(d, SCC_NEWGRF_PRINT_DWORD_SIGNED + c - 0x7B); break;
+			case 0x80: d += Utf8Encode(d, byte80); break;
 			case 0x81: {
 				if (str[0] == '\0' || str[1] == '\0') goto string_end;
 				StringID string;
@@ -463,10 +404,10 @@ char *TranslateTTDPatchCodes(uint32 grfid, uint8 language_id, bool allow_newline
 			}
 			case 0x82:
 			case 0x83:
-			case 0x84: d += Utf8Encode(d, SCC_NEWGRF_PRINT_DATE + c - 0x82); break;
+			case 0x84: d += Utf8Encode(d, SCC_NEWGRF_PRINT_WORD_DATE_LONG + c - 0x82); break;
 			case 0x85: d += Utf8Encode(d, SCC_NEWGRF_DISCARD_WORD);       break;
 			case 0x86: d += Utf8Encode(d, SCC_NEWGRF_ROTATE_TOP_4_WORDS); break;
-			case 0x87: d += Utf8Encode(d, SCC_NEWGRF_PRINT_WORD_VOLUME);  break;
+			case 0x87: d += Utf8Encode(d, SCC_NEWGRF_PRINT_WORD_VOLUME_LONG);  break;
 			case 0x88: d += Utf8Encode(d, SCC_BLUE);    break;
 			case 0x89: d += Utf8Encode(d, SCC_SILVER);  break;
 			case 0x8A: d += Utf8Encode(d, SCC_GOLD);    break;
@@ -509,13 +450,13 @@ char *TranslateTTDPatchCodes(uint32 grfid, uint8 language_id, bool allow_newline
 						d += Utf8Encode(d, SCC_NEWGRF_UNPRINT);
 						d += Utf8Encode(d, *str++);
 						break;
-					case 0x06: d += Utf8Encode(d, SCC_NEWGRF_PRINT_HEX_BYTE);          break;
-					case 0x07: d += Utf8Encode(d, SCC_NEWGRF_PRINT_HEX_WORD);          break;
-					case 0x08: d += Utf8Encode(d, SCC_NEWGRF_PRINT_HEX_DWORD);         break;
+					case 0x06: d += Utf8Encode(d, SCC_NEWGRF_PRINT_BYTE_HEX);          break;
+					case 0x07: d += Utf8Encode(d, SCC_NEWGRF_PRINT_WORD_HEX);          break;
+					case 0x08: d += Utf8Encode(d, SCC_NEWGRF_PRINT_DWORD_HEX);         break;
 					/* 0x09, 0x0A are TTDPatch internal use only string codes. */
-					case 0x0B: d += Utf8Encode(d, SCC_NEWGRF_PRINT_HEX_QWORD);         break;
+					case 0x0B: d += Utf8Encode(d, SCC_NEWGRF_PRINT_QWORD_HEX);         break;
 					case 0x0C: d += Utf8Encode(d, SCC_NEWGRF_PRINT_WORD_STATION_NAME); break;
-					case 0x0D: d += Utf8Encode(d, SCC_NEWGRF_PRINT_WORD_WEIGHT);       break;
+					case 0x0D: d += Utf8Encode(d, SCC_NEWGRF_PRINT_WORD_WEIGHT_LONG);  break;
 					case 0x0E:
 					case 0x0F: {
 						if (str[0] == '\0') goto string_end;
@@ -523,8 +464,8 @@ char *TranslateTTDPatchCodes(uint32 grfid, uint8 language_id, bool allow_newline
 						int index = *str++;
 						int mapped = lm != NULL ? lm->GetMapping(index, code == 0x0E) : -1;
 						if (mapped >= 0) {
-							d += Utf8Encode(d, code == 0x0E ? SCC_GENDER_INDEX : SCC_SETCASE);
-							d += Utf8Encode(d, mapped);
+							d += Utf8Encode(d, code == 0x0E ? SCC_GENDER_INDEX : SCC_SET_CASE);
+							d += Utf8Encode(d, code == 0x0E ? mapped : mapped + 1);
 						}
 						break;
 					}
@@ -576,6 +517,17 @@ char *TranslateTTDPatchCodes(uint32 grfid, uint8 language_id, bool allow_newline
 						}
 						break;
 
+					case 0x16:
+					case 0x17:
+					case 0x18:
+					case 0x19:
+					case 0x1A:
+					case 0x1B:
+					case 0x1C:
+					case 0x1D:
+						d += Utf8Encode(d, SCC_NEWGRF_PRINT_DWORD_DATE_LONG + code - 0x16);
+						break;
+
 					default:
 						grfmsg(1, "missing handler for extended format code");
 						break;
@@ -583,21 +535,21 @@ char *TranslateTTDPatchCodes(uint32 grfid, uint8 language_id, bool allow_newline
 				break;
 			}
 
-			case 0x9E: d += Utf8Encode(d, 0x20AC);             break; // Euro
-			case 0x9F: d += Utf8Encode(d, 0x0178);             break; // Y with diaeresis
-			case 0xA0: d += Utf8Encode(d, SCC_UPARROW);        break;
-			case 0xAA: d += Utf8Encode(d, SCC_DOWNARROW);      break;
-			case 0xAC: d += Utf8Encode(d, SCC_CHECKMARK);      break;
-			case 0xAD: d += Utf8Encode(d, SCC_CROSS);          break;
-			case 0xAF: d += Utf8Encode(d, SCC_RIGHTARROW);     break;
-			case 0xB4: d += Utf8Encode(d, SCC_TRAIN);          break;
-			case 0xB5: d += Utf8Encode(d, SCC_LORRY);          break;
-			case 0xB6: d += Utf8Encode(d, SCC_BUS);            break;
-			case 0xB7: d += Utf8Encode(d, SCC_PLANE);          break;
-			case 0xB8: d += Utf8Encode(d, SCC_SHIP);           break;
-			case 0xB9: d += Utf8Encode(d, SCC_SUPERSCRIPT_M1); break;
-			case 0xBC: d += Utf8Encode(d, SCC_SMALLUPARROW);   break;
-			case 0xBD: d += Utf8Encode(d, SCC_SMALLDOWNARROW); break;
+			case 0x9E: d += Utf8Encode(d, 0x20AC);               break; // Euro
+			case 0x9F: d += Utf8Encode(d, 0x0178);               break; // Y with diaeresis
+			case 0xA0: d += Utf8Encode(d, SCC_UP_ARROW);         break;
+			case 0xAA: d += Utf8Encode(d, SCC_DOWN_ARROW);       break;
+			case 0xAC: d += Utf8Encode(d, SCC_CHECKMARK);        break;
+			case 0xAD: d += Utf8Encode(d, SCC_CROSS);            break;
+			case 0xAF: d += Utf8Encode(d, SCC_RIGHT_ARROW);      break;
+			case 0xB4: d += Utf8Encode(d, SCC_TRAIN);            break;
+			case 0xB5: d += Utf8Encode(d, SCC_LORRY);            break;
+			case 0xB6: d += Utf8Encode(d, SCC_BUS);              break;
+			case 0xB7: d += Utf8Encode(d, SCC_PLANE);            break;
+			case 0xB8: d += Utf8Encode(d, SCC_SHIP);             break;
+			case 0xB9: d += Utf8Encode(d, SCC_SUPERSCRIPT_M1);   break;
+			case 0xBC: d += Utf8Encode(d, SCC_SMALL_UP_ARROW);   break;
+			case 0xBD: d += Utf8Encode(d, SCC_SMALL_DOWN_ARROW); break;
 			default:
 				/* Validate any unhandled character */
 				if (!IsValidChar(c, CS_ALPHANUMERAL)) c = '?';
@@ -744,20 +696,12 @@ StringID AddGRFString(uint32 grfid, uint16 stringid, byte langid_to_add, bool ne
 	return (GRFTAB << TABSIZE) + id;
 }
 
-/* Used to remember the grfid that the last retrieved string came from */
-static uint32 _last_grfid = 0;
-
 /**
  * Returns the index for this stringid associated with its grfID
  */
 StringID GetGRFStringID(uint32 grfid, uint16 stringid)
 {
-	uint id;
-
-	/* grfid is zero when we're being called via an include */
-	if (grfid == 0) grfid = _last_grfid;
-
-	for (id = 0; id < _num_grf_texts; id++) {
+	for (uint id = 0; id < _num_grf_texts; id++) {
 		if (_grf_text[id].grfid == grfid && _grf_text[id].stringid == stringid) {
 			return (GRFTAB << TABSIZE) + id;
 		}
@@ -799,9 +743,6 @@ const char *GetGRFStringPtr(uint16 stringid)
 {
 	assert(_grf_text[stringid].grfid != 0);
 
-	/* Remember this grfid in case the string has included text */
-	_last_grfid = _grf_text[stringid].grfid;
-
 	const char *str = GetGRFStringFromGRFText(_grf_text[stringid].textholder);
 	if (str != NULL) return str;
 
@@ -870,12 +811,14 @@ void CleanUpStrings()
 struct TextRefStack {
 	byte stack[0x30];
 	byte position;
+	const GRFFile *grffile;
 	bool used;
 
-	TextRefStack() : used(false) {}
+	TextRefStack() : position(0), grffile(NULL), used(false) {}
 
 	TextRefStack(const TextRefStack &stack) :
 		position(stack.position),
+		grffile(stack.grffile),
 		used(stack.used)
 	{
 		memcpy(this->stack, stack.stack, sizeof(this->stack));
@@ -927,7 +870,14 @@ struct TextRefStack {
 		this->stack[this->position + 1] = GB(word, 8, 8);
 	}
 
-	void ResetStack()  { this->position = 0; this->used = true; }
+	void ResetStack(const GRFFile *grffile)
+	{
+		assert(grffile != NULL);
+		this->position = 0;
+		this->grffile = grffile;
+		this->used = true;
+	}
+
 	void RewindStack() { this->position = 0; }
 };
 
@@ -976,18 +926,19 @@ void RestoreTextRefStackBackup(struct TextRefStack *backup)
  * by calling #StopTextRefStackUsage(), so NewGRF string codes operate on the
  * normal string parameters again.
  *
+ * @param grffile the NewGRF providing the stack data
  * @param numEntries number of entries to copy from the registers
  * @param values values to copy onto the stack; if NULL the temporary NewGRF registers will be used instead
  */
-void StartTextRefStackUsage(byte numEntries, const uint32 *values)
+void StartTextRefStackUsage(const GRFFile *grffile, byte numEntries, const uint32 *values)
 {
 	extern TemporaryStorageArray<int32, 0x110> _temp_store;
 
-	_newgrf_textrefstack.ResetStack();
+	_newgrf_textrefstack.ResetStack(grffile);
 
 	byte *p = _newgrf_textrefstack.stack;
 	for (uint i = 0; i < numEntries; i++) {
-		uint32 value = values != NULL ? values[i] : _temp_store.Get(0x100 + i);
+		uint32 value = values != NULL ? values[i] : _temp_store.GetValue(0x100 + i);
 		for (uint j = 0; j < 32; j += 8) {
 			*p = GB(value, j, 8);
 			p++;
@@ -1012,81 +963,168 @@ void RewindTextRefStack()
  * @param buff  the buffer we're writing to
  * @param str   the string that we need to write
  * @param argv  the OpenTTD stack of values
+ * @param argv_size space on the stack \a argv
+ * @param modify_argv When true, modify the OpenTTD stack.
  * @return the string control code to "execute" now
  */
-uint RemapNewGRFStringControlCode(uint scc, char *buf_start, char **buff, const char **str, int64 *argv)
+uint RemapNewGRFStringControlCode(uint scc, char *buf_start, char **buff, const char **str, int64 *argv, uint argv_size, bool modify_argv)
 {
-	if (_newgrf_textrefstack.used) {
+	switch (scc) {
+		default: break;
+
+		case SCC_NEWGRF_PRINT_DWORD_SIGNED:
+		case SCC_NEWGRF_PRINT_WORD_SIGNED:
+		case SCC_NEWGRF_PRINT_BYTE_SIGNED:
+		case SCC_NEWGRF_PRINT_WORD_UNSIGNED:
+		case SCC_NEWGRF_PRINT_BYTE_HEX:
+		case SCC_NEWGRF_PRINT_WORD_HEX:
+		case SCC_NEWGRF_PRINT_DWORD_HEX:
+		case SCC_NEWGRF_PRINT_QWORD_HEX:
+		case SCC_NEWGRF_PRINT_DWORD_CURRENCY:
+		case SCC_NEWGRF_PRINT_QWORD_CURRENCY:
+		case SCC_NEWGRF_PRINT_WORD_STRING_ID:
+		case SCC_NEWGRF_PRINT_WORD_DATE_LONG:
+		case SCC_NEWGRF_PRINT_DWORD_DATE_LONG:
+		case SCC_NEWGRF_PRINT_WORD_DATE_SHORT:
+		case SCC_NEWGRF_PRINT_DWORD_DATE_SHORT:
+		case SCC_NEWGRF_PRINT_WORD_SPEED:
+		case SCC_NEWGRF_PRINT_WORD_VOLUME_LONG:
+		case SCC_NEWGRF_PRINT_WORD_VOLUME_SHORT:
+		case SCC_NEWGRF_PRINT_WORD_WEIGHT_LONG:
+		case SCC_NEWGRF_PRINT_WORD_WEIGHT_SHORT:
+		case SCC_NEWGRF_PRINT_WORD_POWER:
+		case SCC_NEWGRF_PRINT_WORD_STATION_NAME:
+			if (argv_size < 1) {
+				DEBUG(misc, 0, "Too many NewGRF string parameters.");
+				return 0;
+			}
+			break;
+
+		case SCC_NEWGRF_PRINT_WORD_CARGO_LONG:
+		case SCC_NEWGRF_PRINT_WORD_CARGO_SHORT:
+		case SCC_NEWGRF_PRINT_WORD_CARGO_TINY:
+			if (argv_size < 2) {
+				DEBUG(misc, 0, "Too many NewGRF string parameters.");
+				return 0;
+			}
+			break;
+	}
+
+	if (_newgrf_textrefstack.used && modify_argv) {
 		switch (scc) {
 			default: NOT_REACHED();
-			case SCC_NEWGRF_PRINT_SIGNED_BYTE:    *argv = _newgrf_textrefstack.PopSignedByte();    break;
-			case SCC_NEWGRF_PRINT_SIGNED_WORD:    *argv = _newgrf_textrefstack.PopSignedWord();    break;
-			case SCC_NEWGRF_PRINT_QWORD_CURRENCY: *argv = _newgrf_textrefstack.PopUnsignedQWord(); break;
+			case SCC_NEWGRF_PRINT_BYTE_SIGNED:      *argv = _newgrf_textrefstack.PopSignedByte();    break;
+			case SCC_NEWGRF_PRINT_QWORD_CURRENCY:   *argv = _newgrf_textrefstack.PopSignedQWord();   break;
 
 			case SCC_NEWGRF_PRINT_DWORD_CURRENCY:
-			case SCC_NEWGRF_PRINT_DWORD:          *argv = _newgrf_textrefstack.PopSignedDWord();   break;
+			case SCC_NEWGRF_PRINT_DWORD_SIGNED:     *argv = _newgrf_textrefstack.PopSignedDWord();   break;
 
-			case SCC_NEWGRF_PRINT_HEX_BYTE:       *argv = _newgrf_textrefstack.PopUnsignedByte();  break;
-			case SCC_NEWGRF_PRINT_HEX_DWORD:      *argv = _newgrf_textrefstack.PopUnsignedDWord(); break;
-			case SCC_NEWGRF_PRINT_HEX_QWORD:      *argv = _newgrf_textrefstack.PopSignedQWord(); break;
+			case SCC_NEWGRF_PRINT_BYTE_HEX:         *argv = _newgrf_textrefstack.PopUnsignedByte();  break;
+			case SCC_NEWGRF_PRINT_QWORD_HEX:        *argv = _newgrf_textrefstack.PopUnsignedQWord(); break;
 
-			case SCC_NEWGRF_PRINT_HEX_WORD:
 			case SCC_NEWGRF_PRINT_WORD_SPEED:
-			case SCC_NEWGRF_PRINT_WORD_VOLUME:
-			case SCC_NEWGRF_PRINT_WORD_WEIGHT:
+			case SCC_NEWGRF_PRINT_WORD_VOLUME_LONG:
+			case SCC_NEWGRF_PRINT_WORD_VOLUME_SHORT:
+			case SCC_NEWGRF_PRINT_WORD_SIGNED:      *argv = _newgrf_textrefstack.PopSignedWord();    break;
+
+			case SCC_NEWGRF_PRINT_WORD_HEX:
+			case SCC_NEWGRF_PRINT_WORD_WEIGHT_LONG:
+			case SCC_NEWGRF_PRINT_WORD_WEIGHT_SHORT:
+			case SCC_NEWGRF_PRINT_WORD_POWER:
 			case SCC_NEWGRF_PRINT_WORD_STATION_NAME:
-			case SCC_NEWGRF_PRINT_UNSIGNED_WORD:  *argv = _newgrf_textrefstack.PopUnsignedWord();  break;
+			case SCC_NEWGRF_PRINT_WORD_UNSIGNED:    *argv = _newgrf_textrefstack.PopUnsignedWord();  break;
 
-			case SCC_NEWGRF_PRINT_DATE:
-			case SCC_NEWGRF_PRINT_MONTH_YEAR:     *argv = _newgrf_textrefstack.PopUnsignedWord() + DAYS_TILL_ORIGINAL_BASE_YEAR; break;
+			case SCC_NEWGRF_PRINT_DWORD_DATE_LONG:
+			case SCC_NEWGRF_PRINT_DWORD_DATE_SHORT:
+			case SCC_NEWGRF_PRINT_DWORD_HEX:        *argv = _newgrf_textrefstack.PopUnsignedDWord(); break;
 
-			case SCC_NEWGRF_DISCARD_WORD:         _newgrf_textrefstack.PopUnsignedWord(); break;
+			case SCC_NEWGRF_PRINT_WORD_DATE_LONG:
+			case SCC_NEWGRF_PRINT_WORD_DATE_SHORT:  *argv = _newgrf_textrefstack.PopUnsignedWord() + DAYS_TILL_ORIGINAL_BASE_YEAR; break;
 
-			case SCC_NEWGRF_ROTATE_TOP_4_WORDS:   _newgrf_textrefstack.RotateTop4Words(); break;
-			case SCC_NEWGRF_PUSH_WORD:            _newgrf_textrefstack.PushWord(Utf8Consume(str)); break;
-			case SCC_NEWGRF_UNPRINT:              *buff = max(*buff - Utf8Consume(str), buf_start); break;
+			case SCC_NEWGRF_DISCARD_WORD:           _newgrf_textrefstack.PopUnsignedWord(); break;
 
-			case SCC_NEWGRF_PRINT_STRING_ID:
-				*argv = TTDPStringIDToOTTDStringIDMapping(_newgrf_textrefstack.PopUnsignedWord());
+			case SCC_NEWGRF_ROTATE_TOP_4_WORDS:     _newgrf_textrefstack.RotateTop4Words(); break;
+			case SCC_NEWGRF_PUSH_WORD:              _newgrf_textrefstack.PushWord(Utf8Consume(str)); break;
+			case SCC_NEWGRF_UNPRINT:                *buff = max(*buff - Utf8Consume(str), buf_start); break;
+
+			case SCC_NEWGRF_PRINT_WORD_CARGO_LONG:
+			case SCC_NEWGRF_PRINT_WORD_CARGO_SHORT:
+			case SCC_NEWGRF_PRINT_WORD_CARGO_TINY:
+				argv[0] = GetCargoTranslation(_newgrf_textrefstack.PopUnsignedWord(), _newgrf_textrefstack.grffile);
+				argv[1] = _newgrf_textrefstack.PopUnsignedWord();
+				break;
+
+			case SCC_NEWGRF_PRINT_WORD_STRING_ID:
+				*argv = MapGRFStringID(_newgrf_textrefstack.grffile->grfid, _newgrf_textrefstack.PopUnsignedWord());
+				break;
+		}
+	} else {
+		/* Consume additional parameter characters */
+		switch (scc) {
+			default: break;
+
+			case SCC_NEWGRF_PUSH_WORD:
+			case SCC_NEWGRF_UNPRINT:
+				Utf8Consume(str);
 				break;
 		}
 	}
 
 	switch (scc) {
 		default: NOT_REACHED();
-		case SCC_NEWGRF_PRINT_DWORD:
-		case SCC_NEWGRF_PRINT_SIGNED_WORD:
-		case SCC_NEWGRF_PRINT_SIGNED_BYTE:
-		case SCC_NEWGRF_PRINT_UNSIGNED_WORD:
+		case SCC_NEWGRF_PRINT_DWORD_SIGNED:
+		case SCC_NEWGRF_PRINT_WORD_SIGNED:
+		case SCC_NEWGRF_PRINT_BYTE_SIGNED:
+		case SCC_NEWGRF_PRINT_WORD_UNSIGNED:
 			return SCC_COMMA;
 
-		case SCC_NEWGRF_PRINT_HEX_BYTE:
-		case SCC_NEWGRF_PRINT_HEX_WORD:
-		case SCC_NEWGRF_PRINT_HEX_DWORD:
-		case SCC_NEWGRF_PRINT_HEX_QWORD:
+		case SCC_NEWGRF_PRINT_BYTE_HEX:
+		case SCC_NEWGRF_PRINT_WORD_HEX:
+		case SCC_NEWGRF_PRINT_DWORD_HEX:
+		case SCC_NEWGRF_PRINT_QWORD_HEX:
 			return SCC_HEX;
 
 		case SCC_NEWGRF_PRINT_DWORD_CURRENCY:
 		case SCC_NEWGRF_PRINT_QWORD_CURRENCY:
-			return SCC_CURRENCY;
+			return SCC_CURRENCY_LONG;
 
-		case SCC_NEWGRF_PRINT_STRING_ID:
-			return SCC_NEWGRF_PRINT_STRING_ID;
+		case SCC_NEWGRF_PRINT_WORD_STRING_ID:
+			return SCC_NEWGRF_PRINT_WORD_STRING_ID;
 
-		case SCC_NEWGRF_PRINT_DATE:
+		case SCC_NEWGRF_PRINT_WORD_DATE_LONG:
+		case SCC_NEWGRF_PRINT_DWORD_DATE_LONG:
 			return SCC_DATE_LONG;
 
-		case SCC_NEWGRF_PRINT_MONTH_YEAR:
-			return SCC_DATE_TINY;
+		case SCC_NEWGRF_PRINT_WORD_DATE_SHORT:
+		case SCC_NEWGRF_PRINT_DWORD_DATE_SHORT:
+			return SCC_DATE_SHORT;
 
 		case SCC_NEWGRF_PRINT_WORD_SPEED:
 			return SCC_VELOCITY;
 
-		case SCC_NEWGRF_PRINT_WORD_VOLUME:
-			return SCC_VOLUME;
+		case SCC_NEWGRF_PRINT_WORD_VOLUME_LONG:
+			return SCC_VOLUME_LONG;
 
-		case SCC_NEWGRF_PRINT_WORD_WEIGHT:
-			return SCC_WEIGHT;
+		case SCC_NEWGRF_PRINT_WORD_VOLUME_SHORT:
+			return SCC_VOLUME_SHORT;
+
+		case SCC_NEWGRF_PRINT_WORD_WEIGHT_LONG:
+			return SCC_WEIGHT_LONG;
+
+		case SCC_NEWGRF_PRINT_WORD_WEIGHT_SHORT:
+			return SCC_WEIGHT_SHORT;
+
+		case SCC_NEWGRF_PRINT_WORD_POWER:
+			return SCC_POWER;
+
+		case SCC_NEWGRF_PRINT_WORD_CARGO_LONG:
+			return SCC_CARGO_LONG;
+
+		case SCC_NEWGRF_PRINT_WORD_CARGO_SHORT:
+			return SCC_CARGO_SHORT;
+
+		case SCC_NEWGRF_PRINT_WORD_CARGO_TINY:
+			return SCC_CARGO_TINY;
 
 		case SCC_NEWGRF_PRINT_WORD_STATION_NAME:
 			return SCC_STATION_NAME;
