@@ -1,4 +1,4 @@
-/* $Id: win32.cpp 25982 2013-11-13 21:35:44Z rubidium $ */
+/* $Id: win32.cpp 26625 2014-06-02 18:18:35Z frosch $ */
 
 /*
  * This file is part of OpenTTD.
@@ -19,7 +19,7 @@
 #include <fcntl.h>
 #include <regstr.h>
 #include <shlobj.h> /* SHGetFolderPath */
-#include <Shellapi.h>
+#include <shellapi.h>
 #include "win32.h"
 #include "../../core/alloc_func.hpp"
 #include "../../openttd.h"
@@ -448,7 +448,7 @@ int APIENTRY WinMain(HINSTANCE hInstance, HINSTANCE hPrevInstance, LPSTR lpCmdLi
 
 	argc = ParseCommandLine(cmdline, argv, lengthof(argv));
 
-	ttd_main(argc, argv);
+	openttd_main(argc, argv);
 	free(cmdline);
 	return 0;
 }
