@@ -1,4 +1,4 @@
-/* $Id: yapf_ship.cpp 24964 2013-02-03 14:16:45Z rubidium $ */
+/* $Id: yapf_ship.cpp 26117 2013-11-26 10:00:15Z rubidium $ */
 
 /*
  * This file is part of OpenTTD.
@@ -167,7 +167,7 @@ public:
 		/* base tile cost depending on distance */
 		int c = IsDiagonalTrackdir(n.GetTrackdir()) ? YAPF_TILE_LENGTH : YAPF_TILE_CORNER_LENGTH;
 		/* additional penalty for curves */
-		if (n.m_parent != NULL && n.GetTrackdir() != NextTrackdir(n.m_parent->GetTrackdir())) {
+		if (n.GetTrackdir() != NextTrackdir(n.m_parent->GetTrackdir())) {
 			/* new trackdir does not match the next one when going straight */
 			c += YAPF_TILE_LENGTH;
 		}
