@@ -1,4 +1,4 @@
-/* $Id: afterload.cpp 23254 2011-11-18 20:52:13Z rubidium $ */
+/* $Id: afterload.cpp 26820 2014-09-14 15:24:39Z frosch $ */
 
 /*
  * This file is part of OpenTTD.
@@ -13,7 +13,6 @@
 #include "../void_map.h"
 #include "../signs_base.h"
 #include "../depot_base.h"
-#include "../window_func.h"
 #include "../fios.h"
 #include "../gamelog_internal.h"
 #include "../network/network.h"
@@ -52,14 +51,14 @@
 #include "../core/backup_type.hpp"
 #include "../smallmap_gui.h"
 #include "../news_func.h"
+#include "../order_backup.h"
+#include "../error.h"
 
-#include "table/strings.h"
 
 #include "saveload_internal.h"
 
 #include <signal.h>
 
-extern StringID _switch_mode_errorstr;
 extern Company *DoStartupNewCompany(bool is_ai, CompanyID company = INVALID_COMPANY);
 
 /**
@@ -76,7 +75,7 @@ void SetWaterClassDependingOnSurroundings(TileIndex t, bool include_invalid_wate
 {
 	/* If the slope is not flat, we always assume 'land' (if allowed). Also for one-corner-raised-shores.
 	 * Note: Wrt. autosloping under industry tiles this is the most fool-proof behaviour. */
-	if (GetTileSlope(t, NULL) != SLOPE_FLAT) {
+	if (!IsTileFlat(t)) {
 		if (include_invalid_water_class) {
 			SetWaterClass(t, WATER_CLASS_INVALID);
 			return;
@@ -250,9 +249,38 @@ static void InitializeWindowsAndCaches()
 		}
 	}
 
+	/* Count number of objects per type */
+	Object *o;
+	FOR_ALL_OBJECTS(o) {
+		Object::IncTypeCount(o->type);
+	}
+
+	/* Identify owners of persistent storage arrays */
+	Industry *i;
+	FOR_ALL_INDUSTRIES(i) {
+		if (i->psa != NULL) {
+			i->psa->feature = GSF_INDUSTRIES;
+			i->psa->tile = i->location.tile;
+		}
+	}
+	Station *s;
+	FOR_ALL_STATIONS(s) {
+		if (s->airport.psa != NULL) {
+			s->airport.psa->feature = GSF_AIRPORTS;
+			s->airport.psa->tile = s->airport.tile;
+		}
+	}
+	Town *t;
+	FOR_ALL_TOWNS(t) {
+		for (std::list<PersistentStorage *>::iterator it = t->psa_list.begin(); it != t->psa_list.end(); ++it) {
+			(*it)->feature = GSF_FAKE_TOWNS;
+			(*it)->tile = t->xy;
+		}
+	}
+
 	RecomputePrices();
 
-	SetCachedEngineCounts();
+	GroupStatistics::UpdateAfterLoad();
 
 	Station::RecomputeIndustriesNearForAll();
 	RebuildSubsidisedSourceAndDestinationCache();
@@ -457,12 +485,12 @@ static uint FixVehicleInclination(Vehicle *v, Direction dir)
 		case INVALID_DIR: break;
 		default: NOT_REACHED();
 	}
-	byte entry_z = GetSlopeZ(entry_x, entry_y);
+	byte entry_z = GetSlopePixelZ(entry_x, entry_y);
 
 	/* Compute middle of the tile. */
-	int middle_x = (v->x_pos & ~TILE_UNIT_MASK) + HALF_TILE_SIZE;
-	int middle_y = (v->y_pos & ~TILE_UNIT_MASK) + HALF_TILE_SIZE;
-	byte middle_z = GetSlopeZ(middle_x, middle_y);
+	int middle_x = (v->x_pos & ~TILE_UNIT_MASK) + TILE_SIZE / 2;
+	int middle_y = (v->y_pos & ~TILE_UNIT_MASK) + TILE_SIZE / 2;
+	byte middle_z = GetSlopePixelZ(middle_x, middle_y);
 
 	/* middle_z == entry_z, no height change. */
 	if (middle_z == entry_z) return 0;
@@ -485,6 +513,10 @@ bool AfterLoadGame()
 
 	TileIndex map_size = MapSize();
 
+	extern TileIndex _cur_tileloop_tile; // From landscape.cpp.
+	/* The LFSR used in RunTileLoop iteration cannot have a zeroed state, make it non-zeroed. */
+	if (_cur_tileloop_tile == 0) _cur_tileloop_tile = 1;
+
 	if (IsSavegameVersionBefore(98)) GamelogOldver();
 
 	GamelogTestRevision();
@@ -621,8 +653,8 @@ bool AfterLoadGame()
 	}
 
 	switch (gcf_res) {
-		case GLC_COMPATIBLE: _switch_mode_errorstr = STR_NEWGRF_COMPATIBLE_LOAD_WARNING; break;
-		case GLC_NOT_FOUND:  _switch_mode_errorstr = STR_NEWGRF_DISABLED_WARNING; _pause_mode = PM_PAUSED_ERROR; break;
+		case GLC_COMPATIBLE: ShowErrorMessage(STR_NEWGRF_COMPATIBLE_LOAD_WARNING, INVALID_STRING_ID, WL_CRITICAL); break;
+		case GLC_NOT_FOUND:  ShowErrorMessage(STR_NEWGRF_DISABLED_WARNING, INVALID_STRING_ID, WL_CRITICAL); _pause_mode = PM_PAUSED_ERROR; break;
 		default: break;
 	}
 
@@ -633,8 +665,45 @@ bool AfterLoadGame()
 	 * must be done before loading sprites as some newgrfs check it */
 	SetDate(_date, _date_fract);
 
-	/* Force dynamic engines off when loading older savegames */
-	if (IsSavegameVersionBefore(95)) _settings_game.vehicle.dynamic_engines = 0;
+	/*
+	 * Force the old behaviour for compatibility reasons with old savegames. As new
+	 * settings can only be loaded from new savegames loading old savegames with new
+	 * versions of OpenTTD will normally initialize settings newer than the savegame
+	 * version with "new game" defaults which the player can define to their liking.
+	 * For some settings we override that to keep the behaviour the same as when the
+	 * game was saved.
+	 *
+	 * Note that there is no non-stop in here. This is because the setting could have
+	 * either value in TTDPatch. To convert it properly the user has to make sure the
+	 * right value has been chosen in the settings. Otherwise we will be converting
+	 * it incorrectly in half of the times without a means to correct that.
+	 */
+	if (IsSavegameVersionBefore(4, 2)) _settings_game.station.modified_catchment = false;
+	if (IsSavegameVersionBefore(6, 1)) _settings_game.pf.forbid_90_deg = false;
+	if (IsSavegameVersionBefore(21))   _settings_game.vehicle.train_acceleration_model = 0;
+	if (IsSavegameVersionBefore(90))   _settings_game.vehicle.plane_speed = 4;
+	if (IsSavegameVersionBefore(95))   _settings_game.vehicle.dynamic_engines = 0;
+	if (IsSavegameVersionBefore(96))   _settings_game.economy.station_noise_level = false;
+	if (IsSavegameVersionBefore(133)) {
+		_settings_game.vehicle.roadveh_acceleration_model = 0;
+		_settings_game.vehicle.train_slope_steepness = 3;
+	}
+	if (IsSavegameVersionBefore(134))  _settings_game.economy.feeder_payment_share = 75;
+	if (IsSavegameVersionBefore(138))  _settings_game.vehicle.plane_crashes = 2;
+	if (IsSavegameVersionBefore(139))  _settings_game.vehicle.roadveh_slope_steepness = 7;
+	if (IsSavegameVersionBefore(143))  _settings_game.economy.allow_town_level_crossings = true;
+	if (IsSavegameVersionBefore(159)) {
+		_settings_game.vehicle.max_train_length = 50;
+		_settings_game.construction.max_bridge_length = 64;
+		_settings_game.construction.max_tunnel_length = 64;
+	}
+	if (IsSavegameVersionBefore(166))  _settings_game.economy.infrastructure_maintenance = false;
+	if (IsSavegameVersionBefore(183)) {
+		_settings_game.linkgraph.distribution_pax = DT_MANUAL;
+		_settings_game.linkgraph.distribution_mail = DT_MANUAL;
+		_settings_game.linkgraph.distribution_armoured = DT_MANUAL;
+		_settings_game.linkgraph.distribution_default = DT_MANUAL;
+	}
 
 	/* Load the sprites */
 	GfxLoadSprites();
@@ -682,7 +751,7 @@ bool AfterLoadGame()
 	}
 
 	/* make sure there is a town in the game */
-	if (_game_mode == GM_NORMAL && !ClosestTownFromTile(0, UINT_MAX)) {
+	if (_game_mode == GM_NORMAL && Town::GetNumItems() == 0) {
 		SetSaveLoadError(STR_ERROR_NO_TOWN_IN_SCENARIO);
 		/* Restore the signals */
 		ResetSignalHandlers();
@@ -697,10 +766,12 @@ bool AfterLoadGame()
 
 	/* If Load Scenario / New (Scenario) Game is used,
 	 *  a company does not exist yet. So create one here.
-	 * 1 exeption: network-games. Those can have 0 companies
-	 *   But this exeption is not true for non dedicated network_servers! */
+	 * 1 exception: network-games. Those can have 0 companies
+	 *   But this exception is not true for non-dedicated network servers! */
 	if (!Company::IsValidID(COMPANY_FIRST) && (!_networking || (_networking && _network_server && !_network_dedicated))) {
 		DoStartupNewCompany(false);
+		Company *c = Company::Get(COMPANY_FIRST);
+		c->settings = _settings_client.company;
 	}
 
 	/* Fix the cache for cargo payments. */
@@ -803,7 +874,7 @@ bool AfterLoadGame()
 
 					case STATION_OILRIG: {
 						/* Very old savegames sometimes have phantom oil rigs, i.e.
-						 * an oil rig which got shut down, but not completly removed from
+						 * an oil rig which got shut down, but not completely removed from
 						 * the map
 						 */
 						TileIndex t1 = TILE_ADDXY(t, 0, 1);
@@ -1059,7 +1130,7 @@ bool AfterLoadGame()
 						if (GB(_m[t].m5, 3, 2) == 0) {
 							MakeClear(t, CLEAR_GRASS, 3);
 						} else {
-							if (GetTileSlope(t, NULL) != SLOPE_FLAT) {
+							if (!IsTileFlat(t)) {
 								MakeShore(t);
 							} else {
 								if (GetTileOwner(t) == OWNER_WATER) {
@@ -1095,7 +1166,7 @@ bool AfterLoadGame()
 					case DIAGDIR_SW: if ((v->x_pos & 0xF) != TILE_SIZE - 1) continue; break;
 					case DIAGDIR_NW: if ((v->y_pos & 0xF) !=  0)            continue; break;
 				}
-			} else if (v->z_pos > GetSlopeZ(v->x_pos, v->y_pos)) {
+			} else if (v->z_pos > GetSlopePixelZ(v->x_pos, v->y_pos)) {
 				v->tile = GetNorthernBridgeEnd(v->tile);
 			} else {
 				continue;
@@ -1151,7 +1222,7 @@ bool AfterLoadGame()
 		}
 
 		FOR_ALL_TRAINS(v) {
-			if (v->IsFrontEngine() || v->IsFreeWagon()) v->ConsistChanged(true);
+			if (v->IsFrontEngine() || v->IsFreeWagon()) v->ConsistChanged(CCF_TRACK);
 		}
 
 	}
@@ -1178,11 +1249,15 @@ bool AfterLoadGame()
 			switch (GetTileType(t)) {
 				case MP_RAILWAY:
 					if (HasSignals(t)) {
+						/* Original signal type/variant was stored in m4 but since saveload
+						 * version 48 they are in m2. The bits has been already moved to m2
+						 * (see the code somewhere above) so don't use m4, use m2 instead. */
+
 						/* convert PBS signals to combo-signals */
-						if (HasBit(_m[t].m2, 2)) SetSignalType(t, TRACK_X, SIGTYPE_COMBO);
+						if (HasBit(_m[t].m2, 2)) SB(_m[t].m2, 0, 2, SIGTYPE_COMBO);
 
 						/* move the signal variant back */
-						SetSignalVariant(t, TRACK_X, HasBit(_m[t].m2, 3) ? SIG_SEMAPHORE : SIG_ELECTRIC);
+						SB(_m[t].m2, 2, 1, HasBit(_m[t].m2, 3) ? SIG_SEMAPHORE : SIG_ELECTRIC);
 						ClrBit(_m[t].m2, 3);
 					}
 
@@ -1257,7 +1332,7 @@ bool AfterLoadGame()
 	}
 
 	/* From 32 on we save the industry who made the farmland.
-	 *  To give this prettyness to old savegames, we remove all farmfields and
+	 *  To give this prettiness to old savegames, we remove all farmfields and
 	 *  plant new ones. */
 	if (IsSavegameVersionBefore(32)) {
 		Industry *i;
@@ -1266,10 +1341,6 @@ bool AfterLoadGame()
 			if (IsTileType(t, MP_CLEAR) && IsClearGround(t, CLEAR_FIELDS)) {
 				/* remove fields */
 				MakeClear(t, CLEAR_GRASS, 3);
-			} else if (IsTileType(t, MP_CLEAR) || IsTileType(t, MP_TREES)) {
-				/* remove fences around fields */
-				SetFenceSE(t, 0);
-				SetFenceSW(t, 0);
 			}
 		}
 
@@ -1407,7 +1478,7 @@ bool AfterLoadGame()
 
 	if (IsSavegameVersionBefore(52)) {
 		for (TileIndex t = 0; t < map_size; t++) {
-			if (IsStatueTile(t)) {
+			if (IsTileType(t, MP_OBJECT) && _m[t].m5 == OBJECT_STATUE) {
 				_m[t].m2 = CalcClosestTownFromTile(t)->index;
 			}
 		}
@@ -1466,13 +1537,15 @@ bool AfterLoadGame()
 	}
 
 	if (IsSavegameVersionBefore(64)) {
-		/* copy the signal type/variant and move signal states bits */
+		/* Since now we allow different signal types and variants on a single tile.
+		 * Move signal states to m4 to make room and clone the signal type/variant. */
 		for (TileIndex t = 0; t < map_size; t++) {
 			if (IsTileType(t, MP_RAILWAY) && HasSignals(t)) {
+				/* move signal states */
 				SetSignalStates(t, GB(_m[t].m2, 4, 4));
-				SetSignalVariant(t, INVALID_TRACK, GetSignalVariant(t, TRACK_X));
-				SetSignalType(t, INVALID_TRACK, GetSignalType(t, TRACK_X));
-				ClrBit(_m[t].m2, 7);
+				SB(_m[t].m2, 4, 4, 0);
+				/* clone signal type and variant */
+				SB(_m[t].m2, 4, 3, GB(_m[t].m2, 0, 3));
 			}
 		}
 	}
@@ -1514,7 +1587,7 @@ bool AfterLoadGame()
 	 */
 	if (IsSavegameVersionBefore(83)) {
 		for (TileIndex t = 0; t < map_size; t++) {
-			if (IsTileType(t, MP_WATER) && IsShipDepot(t)) {
+			if (IsShipDepotTile(t)) {
 				_m[t].m4 = (TileHeight(t) == 0) ? OWNER_WATER : OWNER_NONE;
 			}
 		}
@@ -1525,7 +1598,7 @@ bool AfterLoadGame()
 		FOR_ALL_STATIONS(st) {
 			for (CargoID c = 0; c < NUM_CARGO; c++) {
 				st->goods[c].last_speed = 0;
-				if (st->goods[c].cargo.Count() != 0) SetBit(st->goods[c].acceptance_pickup, GoodsEntry::PICKUP);
+				if (st->goods[c].cargo.AvailableCount() != 0) SetBit(st->goods[c].status, GoodsEntry::GES_RATING);
 			}
 		}
 	}
@@ -1669,7 +1742,7 @@ bool AfterLoadGame()
 		 * on its neighbouring tiles. Done after river and canal updates to
 		 * ensure neighbours are correct. */
 		for (TileIndex t = 0; t < map_size; t++) {
-			if (GetTileSlope(t, NULL) != SLOPE_FLAT) continue;
+			if (!IsTileFlat(t)) continue;
 
 			if (IsTileType(t, MP_WATER) && IsLock(t)) SetWaterClassDependingOnSurroundings(t, false);
 			if (IsTileType(t, MP_STATION) && (IsDock(t) || IsBuoy(t))) SetWaterClassDependingOnSurroundings(t, false);
@@ -1759,9 +1832,7 @@ bool AfterLoadGame()
 		RoadVehicle *v;
 		FOR_ALL_ROADVEHICLES(v) {
 			if (v->First() == v && HasBit(EngInfo(v->engine_type)->misc_flags, EF_ROAD_TRAM)) {
-				if (_switch_mode_errorstr == INVALID_STRING_ID || _switch_mode_errorstr == STR_NEWGRF_COMPATIBLE_LOAD_WARNING) {
-					_switch_mode_errorstr = STR_WARNING_LOADGAME_REMOVED_TRAMS;
-				}
+				ShowErrorMessage(STR_WARNING_LOADGAME_REMOVED_TRAMS, INVALID_STRING_ID, WL_CRITICAL);
 				delete v;
 			}
 		}
@@ -1819,7 +1890,7 @@ bool AfterLoadGame()
 					if (HasStationRail(t)) SetRailStationReservation(t, false);
 					break;
 
-				case MP_TUNNELBRIDGE: // Clear PBS reservation on tunnels/birdges
+				case MP_TUNNELBRIDGE: // Clear PBS reservation on tunnels/bridges
 					if (GetTunnelBridgeTransportType(t) == TRANSPORT_RAIL) SetTunnelBridgeReservation(t, false);
 					break;
 
@@ -1905,7 +1976,7 @@ bool AfterLoadGame()
 			if (!IsTileType(t, MP_OBJECT)) continue;
 
 			/* Reordering/generalisation of the object bits. */
-			ObjectType type = GetObjectType(t);
+			ObjectType type = _m[t].m5;
 			SB(_m[t].m6, 2, 4, type == OBJECT_HQ ? GB(_m[t].m3, 2, 3) : 0);
 			_m[t].m3 = type == OBJECT_HQ ? GB(_m[t].m3, 1, 1) | GB(_m[t].m3, 0, 1) << 4 : 0;
 
@@ -1932,7 +2003,7 @@ bool AfterLoadGame()
 
 				if (offset == 0) {
 					/* No offset, so make the object. */
-					ObjectType type = GetObjectType(t);
+					ObjectType type = _m[t].m5;
 					int size = type == OBJECT_HQ ? 2 : 1;
 
 					if (!Object::CanAllocateItem()) {
@@ -2161,15 +2232,10 @@ bool AfterLoadGame()
 
 		/* Simulate the inflation, so we also get the payment inflation */
 		while (_economy.inflation_prices < aimed_inflation) {
-			AddInflation(false);
+			if (AddInflation(false)) break;
 		}
 	}
 
-	if (IsSavegameVersionBefore(127)) {
-		Station *st;
-		FOR_ALL_STATIONS(st) UpdateStationAcceptance(st, false);
-	}
-
 	if (IsSavegameVersionBefore(128)) {
 		const Depot *d;
 		FOR_ALL_DEPOTS(d) {
@@ -2354,7 +2420,7 @@ bool AfterLoadGame()
 	if (IsSavegameVersionBefore(149)) {
 		for (TileIndex t = 0; t < map_size; t++) {
 			if (!IsTileType(t, MP_STATION)) continue;
-			if (!IsBuoy(t) && !IsOilRig(t) && !(IsDock(t) && GetTileSlope(t, NULL) == SLOPE_FLAT)) {
+			if (!IsBuoy(t) && !IsOilRig(t) && !(IsDock(t) && IsTileFlat(t))) {
 				SetWaterClass(t, WATER_CLASS_INVALID);
 			}
 		}
@@ -2393,7 +2459,7 @@ bool AfterLoadGame()
 			if (!IsTunnelTile(vtile)) continue;
 
 			/* Are we actually in this tunnel? Or maybe a lower tunnel? */
-			if (GetSlopeZ(v->x_pos, v->y_pos) != v->z_pos) continue;
+			if (GetSlopePixelZ(v->x_pos, v->y_pos) != v->z_pos) continue;
 
 			/* What way are we going? */
 			const DiagDirection dir = GetTunnelBridgeDirection(vtile);
@@ -2543,7 +2609,7 @@ bool AfterLoadGame()
 				/* In old versions, z_pos was 1 unit lower on bridge heads.
 				 * However, this invalid state could be converted to new savegames
 				 * by loading and saving the game in a new version. */
-				v->z_pos = GetSlopeZ(v->x_pos, v->y_pos);
+				v->z_pos = GetSlopePixelZ(v->x_pos, v->y_pos);
 				DiagDirection dir = GetTunnelBridgeDirection(v->tile);
 				if (v->type == VEH_TRAIN && !(v->vehstatus & VS_CRASHED) &&
 						v->direction != DiagDirToDir(dir)) {
@@ -2557,7 +2623,7 @@ bool AfterLoadGame()
 
 			/* If the vehicle is really above v->tile (not in a wormhole),
 			 * it should have set v->z_pos correctly. */
-			assert(v->tile != TileVirtXY(v->x_pos, v->y_pos) || v->z_pos == GetSlopeZ(v->x_pos, v->y_pos));
+			assert(v->tile != TileVirtXY(v->x_pos, v->y_pos) || v->z_pos == GetSlopePixelZ(v->x_pos, v->y_pos));
 		}
 
 		/* Fill Vehicle::cur_real_order_index */
@@ -2595,20 +2661,295 @@ bool AfterLoadGame()
 		}
 	}
 
+	if (IsSavegameVersionBefore(161)) {
+		/* Before savegame version 161, persistent storages were not stored in a pool. */
+
+		if (!IsSavegameVersionBefore(76)) {
+			Industry *ind;
+			FOR_ALL_INDUSTRIES(ind) {
+				assert(ind->psa != NULL);
+
+				/* Check if the old storage was empty. */
+				bool is_empty = true;
+				for (uint i = 0; i < sizeof(ind->psa->storage); i++) {
+					if (ind->psa->GetValue(i) != 0) {
+						is_empty = false;
+						break;
+					}
+				}
+
+				if (!is_empty) {
+					ind->psa->grfid = _industry_mngr.GetGRFID(ind->type);
+				} else {
+					delete ind->psa;
+					ind->psa = NULL;
+				}
+			}
+		}
+
+		if (!IsSavegameVersionBefore(145)) {
+			Station *st;
+			FOR_ALL_STATIONS(st) {
+				if (!(st->facilities & FACIL_AIRPORT)) continue;
+				assert(st->airport.psa != NULL);
+
+				/* Check if the old storage was empty. */
+				bool is_empty = true;
+				for (uint i = 0; i < sizeof(st->airport.psa->storage); i++) {
+					if (st->airport.psa->GetValue(i) != 0) {
+						is_empty = false;
+						break;
+					}
+				}
+
+				if (!is_empty) {
+					st->airport.psa->grfid = _airport_mngr.GetGRFID(st->airport.type);
+				} else {
+					delete st->airport.psa;
+					st->airport.psa = NULL;
+
+				}
+			}
+		}
+	}
+
+	/* This triggers only when old snow_lines were copied into the snow_line_height. */
+	if (IsSavegameVersionBefore(164) && _settings_game.game_creation.snow_line_height >= MIN_SNOWLINE_HEIGHT * TILE_HEIGHT) {
+		_settings_game.game_creation.snow_line_height /= TILE_HEIGHT;
+	}
+
+	if (IsSavegameVersionBefore(164) && !IsSavegameVersionBefore(32)) {
+		/* We store 4 fences in the field tiles instead of only SE and SW. */
+		for (TileIndex t = 0; t < map_size; t++) {
+			if (!IsTileType(t, MP_CLEAR) && !IsTileType(t, MP_TREES)) continue;
+			if (IsTileType(t, MP_CLEAR) && IsClearGround(t, CLEAR_FIELDS)) continue;
+			uint fence = GB(_m[t].m4, 5, 3);
+			if (fence != 0 && IsTileType(TILE_ADDXY(t, 1, 0), MP_CLEAR) && IsClearGround(TILE_ADDXY(t, 1, 0), CLEAR_FIELDS)) {
+				SetFence(TILE_ADDXY(t, 1, 0), DIAGDIR_NE, fence);
+			}
+			fence = GB(_m[t].m4, 2, 3);
+			if (fence != 0 && IsTileType(TILE_ADDXY(t, 0, 1), MP_CLEAR) && IsClearGround(TILE_ADDXY(t, 0, 1), CLEAR_FIELDS)) {
+				SetFence(TILE_ADDXY(t, 0, 1), DIAGDIR_NW, fence);
+			}
+			SB(_m[t].m4, 2, 3, 0);
+			SB(_m[t].m4, 5, 3, 0);
+		}
+	}
+
+	/* The center of train vehicles was changed, fix up spacing. */
+	if (IsSavegameVersionBefore(164)) FixupTrainLengths();
+
+	if (IsSavegameVersionBefore(165)) {
+		Town *t;
+
+		FOR_ALL_TOWNS(t) {
+			/* Set the default cargo requirement for town growth */
+			switch (_settings_game.game_creation.landscape) {
+				case LT_ARCTIC:
+					if (FindFirstCargoWithTownEffect(TE_FOOD) != NULL) t->goal[TE_FOOD] = TOWN_GROWTH_WINTER;
+					break;
+
+				case LT_TROPIC:
+					if (FindFirstCargoWithTownEffect(TE_FOOD) != NULL) t->goal[TE_FOOD] = TOWN_GROWTH_DESERT;
+					if (FindFirstCargoWithTownEffect(TE_WATER) != NULL) t->goal[TE_WATER] = TOWN_GROWTH_DESERT;
+					break;
+			}
+		}
+	}
+
+	if (IsSavegameVersionBefore(165)) {
+		/* Adjust zoom level to account for new levels */
+		_saved_scrollpos_zoom = _saved_scrollpos_zoom + ZOOM_LVL_SHIFT;
+		_saved_scrollpos_x *= ZOOM_LVL_BASE;
+		_saved_scrollpos_y *= ZOOM_LVL_BASE;
+	}
+
 	/* When any NewGRF has been changed the availability of some vehicles might
 	 * have been changed too. e->company_avail must be set to 0 in that case
 	 * which is done by StartupEngines(). */
 	if (gcf_res != GLC_ALL_GOOD) StartupEngines();
 
+	if (IsSavegameVersionBefore(166)) {
+		/* Update cargo acceptance map of towns. */
+		for (TileIndex t = 0; t < map_size; t++) {
+			if (!IsTileType(t, MP_HOUSE)) continue;
+			Town::Get(GetTownIndex(t))->cargo_accepted.Add(t);
+		}
+
+		Town *town;
+		FOR_ALL_TOWNS(town) {
+			UpdateTownCargoes(town);
+		}
+	}
+
+	/* The road owner of standard road stops was not properly accounted for. */
+	if (IsSavegameVersionBefore(172)) {
+		for (TileIndex t = 0; t < map_size; t++) {
+			if (!IsStandardRoadStopTile(t)) continue;
+			Owner o = GetTileOwner(t);
+			SetRoadOwner(t, ROADTYPE_ROAD, o);
+			SetRoadOwner(t, ROADTYPE_TRAM, o);
+		}
+	}
+
+	if (IsSavegameVersionBefore(175)) {
+		/* Introduced tree planting limit. */
+		Company *c;
+		FOR_ALL_COMPANIES(c) c->tree_limit = _settings_game.construction.tree_frame_burst << 16;
+	}
+
+	if (IsSavegameVersionBefore(177)) {
+		/* Fix too high inflation rates */
+		if (_economy.inflation_prices > MAX_INFLATION) _economy.inflation_prices = MAX_INFLATION;
+		if (_economy.inflation_payment > MAX_INFLATION) _economy.inflation_payment = MAX_INFLATION;
+
+		/* We have to convert the quarters of bankruptcy into months of bankruptcy */
+		FOR_ALL_COMPANIES(c) {
+			c->months_of_bankruptcy = 3 * c->months_of_bankruptcy;
+		}
+	}
+
+	if (IsSavegameVersionBefore(178)) {
+		extern uint8 _old_diff_level;
+		/* Initialise script settings profile */
+		_settings_game.script.settings_profile = IsInsideMM(_old_diff_level, SP_BEGIN, SP_END) ? _old_diff_level : (uint)SP_MEDIUM;
+	}
+
+	if (IsSavegameVersionBefore(182)) {
+		Aircraft *v;
+		/* Aircraft acceleration variable was bonkers */
+		FOR_ALL_AIRCRAFT(v) {
+			if (v->subtype <= AIR_AIRCRAFT) {
+				const AircraftVehicleInfo *avi = AircraftVehInfo(v->engine_type);
+				v->acceleration = avi->acceleration;
+			}
+		}
+
+		/* Blocked tiles could be reserved due to a bug, which causes
+		 * other places to assert upon e.g. station reconstruction. */
+		for (TileIndex t = 0; t < map_size; t++) {
+			if (HasStationTileRail(t) && IsStationTileBlocked(t)) {
+				SetRailStationReservation(t, false);
+			}
+		}
+	}
+
+	if (IsSavegameVersionBefore(184)) {
+		/* The global units configuration is split up in multiple configurations. */
+		extern uint8 _old_units;
+		_settings_game.locale.units_velocity = Clamp(_old_units, 0, 2);
+		_settings_game.locale.units_power    = Clamp(_old_units, 0, 2);
+		_settings_game.locale.units_weight   = Clamp(_old_units, 1, 2);
+		_settings_game.locale.units_volume   = Clamp(_old_units, 1, 2);
+		_settings_game.locale.units_force    = 2;
+		_settings_game.locale.units_height   = Clamp(_old_units, 0, 2);
+	}
+
+	if (IsSavegameVersionBefore(186)) {
+		/* Move ObjectType from map to pool */
+		for (TileIndex t = 0; t < map_size; t++) {
+			if (IsTileType(t, MP_OBJECT)) {
+				Object *o = Object::Get(_m[t].m2);
+				o->type = _m[t].m5;
+				_m[t].m5 = 0; // zero upper bits of (now bigger) ObjectID
+			}
+		}
+	}
+
+	if (IsSavegameVersionBefore(188)) {
+		/* Fix articulated road vehicles.
+		 * Some curves were shorter than other curves.
+		 * Now they have the same length, but that means that trailing articulated parts will
+		 * take longer to go through the curve than the parts in front which already left the courve.
+		 * So, make articulated parts catch up. */
+		RoadVehicle *v;
+		bool roadside = _settings_game.vehicle.road_side == 1;
+		SmallVector<uint, 16> skip_frames;
+		FOR_ALL_ROADVEHICLES(v) {
+			if (!v->IsFrontEngine()) continue;
+			skip_frames.Clear();
+			TileIndex prev_tile = v->tile;
+			uint prev_tile_skip = 0;
+			uint cur_skip = 0;
+			for (RoadVehicle *u = v; u != NULL; u = u->Next()) {
+				if (u->tile != prev_tile) {
+					prev_tile_skip = cur_skip;
+					prev_tile = u->tile;
+				} else {
+					cur_skip = prev_tile_skip;
+				}
+
+				uint *this_skip = skip_frames.Append();
+				*this_skip = prev_tile_skip;
+
+				/* The following 3 curves now take longer than before */
+				switch (u->state) {
+					case 2:
+						cur_skip++;
+						if (u->frame <= (roadside ? 9 : 5)) *this_skip = cur_skip;
+						break;
+
+					case 4:
+						cur_skip++;
+						if (u->frame <= (roadside ? 5 : 9)) *this_skip = cur_skip;
+						break;
+
+					case 5:
+						cur_skip++;
+						if (u->frame <= (roadside ? 4 : 2)) *this_skip = cur_skip;
+						break;
+
+					default:
+						break;
+				}
+			}
+			while (cur_skip > skip_frames[0]) {
+				RoadVehicle *u = v;
+				RoadVehicle *prev = NULL;
+				for (uint *it = skip_frames.Begin(); it != skip_frames.End(); ++it, prev = u, u = u->Next()) {
+					extern bool IndividualRoadVehicleController(RoadVehicle *v, const RoadVehicle *prev);
+					if (*it >= cur_skip) IndividualRoadVehicleController(u, prev);
+				}
+				cur_skip--;
+			}
+		}
+	}
+
+	/*
+	 * Only keep order-backups for network clients.
+	 * If we are a network server or not networking, then we just loaded a previously
+	 * saved-by-server savegame. There are no clients with a backup, so clear it.
+	 * Furthermore before savegame version 192 the actual content was always corrupt.
+	 */
+	if (!_networking || _network_server || IsSavegameVersionBefore(192)) {
+		/* Note: We cannot use CleanPool since that skips part of the destructor
+		 * and then leaks un-reachable Orders in the order pool. */
+		OrderBackup *ob;
+		FOR_ALL_ORDER_BACKUPS(ob) {
+			delete ob;
+		}
+	}
+
+
+	/* Station acceptance is some kind of cache */
+	if (IsSavegameVersionBefore(127)) {
+		Station *st;
+		FOR_ALL_STATIONS(st) UpdateStationAcceptance(st, false);
+	}
+
 	/* Road stops is 'only' updating some caches */
 	AfterLoadRoadStops();
 	AfterLoadLabelMaps();
+	AfterLoadCompanyStats();
+	AfterLoadStoryBook();
 
 	GamelogPrintDebug(1);
 
 	InitializeWindowsAndCaches();
 	/* Restore the signals */
 	ResetSignalHandlers();
+
+	AfterLoadLinkGraphs();
 	return true;
 }
 
@@ -2627,18 +2968,22 @@ void ReloadNewGRFData()
 	LoadStringWidthTable();
 	RecomputePrices();
 	/* reload vehicles */
-	ResetVehiclePosHash();
+	ResetVehicleHash();
 	AfterLoadVehicles(false);
 	StartupEngines();
-	SetCachedEngineCounts();
+	GroupStatistics::UpdateAfterLoad();
 	/* update station graphics */
 	AfterLoadStations();
+	/* Update company statistics. */
+	AfterLoadCompanyStats();
 	/* Check and update house and town values */
 	UpdateHousesAndTowns();
 	/* Delete news referring to no longer existing entities */
 	DeleteInvalidEngineNews();
 	/* Update livery selection windows */
 	for (CompanyID i = COMPANY_FIRST; i < MAX_COMPANIES; i++) InvalidateWindowData(WC_COMPANY_COLOUR, i);
+	/* Update company infrastructure counts. */
+	InvalidateWindowClassesData(WC_COMPANY_INFRASTRUCTURE);
 	/* redraw the whole screen */
 	MarkWholeScreenDirty();
 	CheckTrainsLengths();
