@@ -1,4 +1,4 @@
-/* $Id: strings_sl.cpp 22747 2011-08-14 15:23:10Z rubidium $ */
+/* $Id: strings_sl.cpp 22957 2011-09-23 19:27:15Z rubidium $ */
 
 /*
  * This file is part of OpenTTD.
@@ -29,8 +29,8 @@ StringID RemapOldStringID(StringID s)
 	switch (s) {
 		case 0x0006: return STR_SV_EMPTY;
 		case 0x7000: return STR_SV_UNNAMED;
-		case 0x70E4: return SPECSTR_PLAYERNAME_ENGLISH;
-		case 0x70E9: return SPECSTR_PLAYERNAME_ENGLISH;
+		case 0x70E4: return SPECSTR_COMPANY_NAME_START;
+		case 0x70E9: return SPECSTR_COMPANY_NAME_START;
 		case 0x8864: return STR_SV_TRAIN_NAME;
 		case 0x902B: return STR_SV_ROAD_VEHICLE_NAME;
 		case 0x9830: return STR_SV_SHIP_NAME;
@@ -118,6 +118,9 @@ void InitializeOldNames()
 	_old_name_array = CallocT<char>(NUM_OLD_STRINGS * LEN_OLD_STRINGS); // 200 * 24 would be enough for TTO savegames
 }
 
+/**
+ * Load the NAME chunk.
+ */
 static void Load_NAME()
 {
 	int index;
@@ -132,6 +135,7 @@ static void Load_NAME()
 	}
 }
 
+/** Chunk handlers related to strings. */
 extern const ChunkHandler _name_chunk_handlers[] = {
 	{ 'NAME', NULL, Load_NAME, NULL, NULL, CH_ARRAY | CH_LAST},
 };
