@@ -1,4 +1,4 @@
-/* $Id: signs_type.h 21797 2011-01-14 22:08:40Z rubidium $ */
+/* $Id: signs_type.h 22406 2011-05-01 19:51:52Z rubidium $ */
 
 /*
  * This file is part of OpenTTD.
@@ -12,12 +12,12 @@
 #ifndef SIGNS_TYPE_H
 #define SIGNS_TYPE_H
 
+/** The type of the IDs of signs. */
 typedef uint16 SignID;
 struct Sign;
 
-static const SignID INVALID_SIGN = 0xFFFF;
+static const SignID INVALID_SIGN = 0xFFFF; ///< Sentinel for an invalid sign.
 
-static const uint MAX_LENGTH_SIGN_NAME_CHARS  =  32; ///< The maximum length of a sign name in characters including '\0'
-static const uint MAX_LENGTH_SIGN_NAME_PIXELS = 255; ///< The maximum length of a sign name in pixels
+static const uint MAX_LENGTH_SIGN_NAME_CHARS = 32; ///< The maximum length of a sign name in characters including '\0'
 
 #endif /* SIGNS_TYPE_H */
