@@ -1,4 +1,4 @@
-/* $Id: station_func.h 20674 2010-08-28 20:15:45Z alberth $ */
+/* $Id: station_func.h 25435 2013-06-23 08:29:28Z fonsinchen $ */
 
 /*
  * This file is part of OpenTTD.
@@ -12,12 +12,12 @@
 #ifndef STATION_FUNC_H
 #define STATION_FUNC_H
 
-#include "station_type.h"
 #include "sprite.h"
 #include "rail_type.h"
 #include "road_type.h"
-#include "cargo_type.h"
-#include "company_type.h"
+#include "vehicle_type.h"
+#include "economy_func.h"
+#include "rail.h"
 
 void ModifyStationRatingAround(TileIndex tile, Owner owner, int amount, uint radius);
 
@@ -41,9 +41,27 @@ void DeleteOilRig(TileIndex t);
 /* Check if a rail station tile is traversable. */
 bool IsStationTileBlocked(TileIndex tile);
 
-/* Check if a rail station tile is electrifiable. */
-bool IsStationTileElectrifiable(TileIndex tile);
+bool CanStationTileHavePylons(TileIndex tile);
+bool CanStationTileHaveWires(TileIndex tile);
 
 void UpdateAirportsNoise();
 
+bool SplitGroundSpriteForOverlay(const TileInfo *ti, SpriteID *ground, RailTrackOffset *overlay_offset);
+
+void IncreaseStats(Station *st, const Vehicle *v, StationID next_station_id);
+void IncreaseStats(Station *st, CargoID cargo, StationID next_station_id, uint capacity, uint usage);
+void RerouteCargo(Station *st, CargoID c, StationID avoid, StationID avoid2);
+
+/**
+ * Calculates the maintenance cost of a number of station tiles.
+ * @param num Number of station tiles.
+ * @return Total cost.
+ */
+static inline Money StationMaintenanceCost(uint32 num)
+{
+	return (_price[PR_INFRASTRUCTURE_STATION] * num * (1 + IntSqrt(num))) >> 7; // 7 bits scaling.
+}
+
+Money AirportMaintenanceCost(Owner owner);
+
 #endif /* STATION_FUNC_H */
