@@ -1,4 +1,4 @@
-/* $Id: strgen.cpp 25204 2013-04-24 17:54:43Z rubidium $ */
+/* $Id: strgen.cpp 26544 2014-04-29 18:41:19Z frosch $ */
 
 /*
  * This file is part of OpenTTD.
@@ -122,7 +122,7 @@ struct FileStringReader : StringReader {
 
 	/* virtual */ char *ReadLine(char *buffer, size_t size)
 	{
-		return fgets(buffer, size, this->fh);
+		return fgets(buffer, ClampToU16(size), this->fh);
 	}
 
 	/* virtual */ void HandlePragma(char *str);
@@ -290,6 +290,12 @@ struct HeaderFileWriter : HeaderWriter, FileWriter {
 		fprintf(this->fh, "#define TABLE_STRINGS_H\n");
 	}
 
+	/** Free the filename. */
+	~HeaderFileWriter()
+	{
+		free(real_filename);
+	}
+
 	void WriteStringID(const char *name, int stringid)
 	{
 		if (prev + 1 != stringid) fprintf(this->fh, "\n");
@@ -347,7 +353,9 @@ struct LanguageFileWriter : LanguageWriter, FileWriter {
 
 	void Finalise()
 	{
-		fputc(0, this->fh);
+		if (fputc(0, this->fh) == EOF) {
+			error("Could not write to %s", this->filename);
+		}
 		this->FileWriter::Finalise();
 	}
 
@@ -362,10 +370,12 @@ struct LanguageFileWriter : LanguageWriter, FileWriter {
 /** Multi-OS mkdirectory function */
 static inline void ottd_mkdir(const char *directory)
 {
+	/* Ignore directory creation errors; they'll surface later on, and most
+	 * of the time they are 'directory already exists' errors anyhow. */
 #if defined(WIN32) || defined(__WATCOMC__)
-		mkdir(directory);
+	mkdir(directory);
 #else
-		mkdir(directory, 0755);
+	mkdir(directory, 0755);
 #endif
 }
 
@@ -427,7 +437,7 @@ int CDECL main(int argc, char *argv[])
 
 		switch (i) {
 			case 'v':
-				puts("$Revision: 25204 $");
+				puts("$Revision: 26544 $");
 				return 0;
 
 			case 'C':
@@ -472,7 +482,7 @@ int CDECL main(int argc, char *argv[])
 
 			case 'h':
 				puts(
-					"strgen - $Revision: 25204 $\n"
+					"strgen - $Revision: 26544 $\n"
 					" -v | --version    print version information and exit\n"
 					" -t | --todo       replace any untranslated strings with '<TODO>'\n"
 					" -w | --warning    print a warning for any untranslated strings\n"
@@ -551,7 +561,7 @@ int CDECL main(int argc, char *argv[])
 				/* rename the .txt (input-extension) to .lng */
 				r = strrchr(pathbuf, '.');
 				if (r == NULL || strcmp(r, ".txt") != 0) r = strchr(pathbuf, '\0');
-				ttd_strlcpy(r, ".lng", (size_t)(r - pathbuf));
+				strecpy(r, ".lng", lastof(pathbuf));
 
 				LanguageFileWriter writer(pathbuf);
 				writer.WriteLang(data);
