@@ -1,4 +1,4 @@
-/* $Id: terraform_gui.h 18781 2010-01-11 18:46:09Z rubidium $ */
+/* $Id: terraform_gui.h 21608 2010-12-23 14:24:34Z alberth $ */
 
 /*
  * This file is part of OpenTTD.
@@ -15,8 +15,6 @@
 #include "window_type.h"
 
 Window *ShowTerraformToolbar(Window *link = NULL);
-void ShowTerraformToolbarWithTool(uint16 key, uint16 keycode);
 Window *ShowEditorTerraformToolbar();
-void ShowEditorTerraformToolbarWithTool(uint16 key, uint16 keycode);
 
-#endif /* GUI_H */
+#endif /* TERRAFORM_GUI_H */
