@@ -1,4 +1,4 @@
-/* $Id: timetable_gui.cpp 25340 2013-06-09 09:38:21Z rubidium $ */
+/* $Id: timetable_gui.cpp 26653 2014-06-17 19:14:59Z frosch $ */
 
 /*
  * This file is part of OpenTTD.
@@ -53,18 +53,6 @@ void SetTimetableParams(int param1, int param2, Ticks ticks)
 	}
 }
 
-/**
- * Sets the arrival or departure string and parameters.
- * @param param1 the first DParam to fill
- * @param param2 the second DParam to fill
- * @param ticks  the number of ticks to 'draw'
- */
-static void SetArrivalDepartParams(int param1, int param2, Ticks ticks)
-{
-	SetDParam(param1, STR_JUST_DATE_TINY);
-	SetDParam(param2, _date + (ticks / DAY_TICKS));
-}
-
 /**
  * Check whether it is possible to determine how long the order takes.
  * @param order the order to check.
@@ -164,16 +152,16 @@ struct TimetableWindow : Window {
 	Scrollbar *vscroll;
 	bool query_is_speed_query; ///< The currently open query window is a speed query and not a time query.
 
-	TimetableWindow(const WindowDesc *desc, WindowNumber window_number) :
-			Window(),
+	TimetableWindow(WindowDesc *desc, WindowNumber window_number) :
+			Window(desc),
 			sel_index(-1),
 			vehicle(Vehicle::Get(window_number)),
 			show_expected(true)
 	{
-		this->CreateNestedTree(desc);
+		this->CreateNestedTree();
 		this->vscroll = this->GetScrollbar(WID_VT_SCROLLBAR);
 		this->UpdateSelectionStates();
-		this->FinishInitNested(desc, window_number);
+		this->FinishInitNested(window_number);
 
 		this->owner = this->vehicle->owner;
 	}
@@ -328,9 +316,9 @@ struct TimetableWindow : Window {
 			this->SetWidgetDisabledState(WID_VT_CLEAR_SPEED, disable_speed);
 			this->SetWidgetDisabledState(WID_VT_SHARED_ORDER_LIST, !v->IsOrderListShared());
 
-			this->EnableWidget(WID_VT_START_DATE);
-			this->EnableWidget(WID_VT_RESET_LATENESS);
-			this->EnableWidget(WID_VT_AUTOFILL);
+			this->SetWidgetDisabledState(WID_VT_START_DATE, v->orders.list == NULL);
+			this->SetWidgetDisabledState(WID_VT_RESET_LATENESS, v->orders.list == NULL);
+			this->SetWidgetDisabledState(WID_VT_AUTOFILL, v->orders.list == NULL);
 		} else {
 			this->DisableWidget(WID_VT_START_DATE);
 			this->DisableWidget(WID_VT_CHANGE_TIME);
@@ -447,18 +435,20 @@ struct TimetableWindow : Window {
 						if (arr_dep[i / 2].arrival != INVALID_TICKS) {
 							DrawString(abbr_left, abbr_right, y, STR_TIMETABLE_ARRIVAL_ABBREVIATION, i == selected ? TC_WHITE : TC_BLACK);
 							if (this->show_expected && i / 2 == earlyID) {
-								SetArrivalDepartParams(0, 1, arr_dep[i / 2].arrival);
-								DrawString(time_left, time_right, y, STR_GREEN_STRING, i == selected ? TC_WHITE : TC_BLACK);
+								SetDParam(0, _date + arr_dep[i / 2].arrival / DAY_TICKS);
+								DrawString(time_left, time_right, y, STR_JUST_DATE_TINY, TC_GREEN);
 							} else {
-								SetArrivalDepartParams(0, 1, arr_dep[i / 2].arrival + offset);
-								DrawString(time_left, time_right, y, show_late ? STR_RED_STRING : STR_JUST_STRING, i == selected ? TC_WHITE : TC_BLACK);
+								SetDParam(0, _date + (arr_dep[i / 2].arrival + offset) / DAY_TICKS);
+								DrawString(time_left, time_right, y, STR_JUST_DATE_TINY,
+										show_late ? TC_RED : i == selected ? TC_WHITE : TC_BLACK);
 							}
 						}
 					} else {
 						if (arr_dep[i / 2].departure != INVALID_TICKS) {
 							DrawString(abbr_left, abbr_right, y, STR_TIMETABLE_DEPARTURE_ABBREVIATION, i == selected ? TC_WHITE : TC_BLACK);
-							SetArrivalDepartParams(0, 1, arr_dep[i/2].departure + offset);
-							DrawString(time_left, time_right, y, show_late ? STR_RED_STRING : STR_JUST_STRING, i == selected ? TC_WHITE : TC_BLACK);
+							SetDParam(0, _date + (arr_dep[i/2].departure + offset) / DAY_TICKS);
+							DrawString(time_left, time_right, y, STR_JUST_DATE_TINY,
+									show_late ? TC_RED : i == selected ? TC_WHITE : TC_BLACK);
 						}
 					}
 					y += FONT_HEIGHT_NORMAL;
@@ -525,7 +515,7 @@ struct TimetableWindow : Window {
 			}
 
 			case WID_VT_START_DATE: // Change the date that the timetable starts.
-				ShowSetDateWindow(this, v->index, _date, _cur_year, _cur_year + 15, ChangeTimetableStartCallback);
+				ShowSetDateWindow(this, v->index | (v->orders.list->IsCompleteTimetable() && _ctrl_pressed ? 1U << 20 : 0), _date, _cur_year, _cur_year + 15, ChangeTimetableStartCallback);
 				break;
 
 			case WID_VT_CHANGE_TIME: { // "Wait For" button.
@@ -650,6 +640,7 @@ static const NWidgetPart _nested_timetable_widgets[] = {
 		NWidget(WWT_CAPTION, COLOUR_GREY, WID_VT_CAPTION), SetDataTip(STR_TIMETABLE_TITLE, STR_TOOLTIP_WINDOW_TITLE_DRAG_THIS),
 		NWidget(WWT_PUSHTXTBTN, COLOUR_GREY, WID_VT_ORDER_VIEW), SetMinimalSize(61, 14), SetDataTip( STR_TIMETABLE_ORDER_VIEW, STR_TIMETABLE_ORDER_VIEW_TOOLTIP),
 		NWidget(WWT_SHADEBOX, COLOUR_GREY),
+		NWidget(WWT_DEFSIZEBOX, COLOUR_GREY),
 		NWidget(WWT_STICKYBOX, COLOUR_GREY),
 	EndContainer(),
 	NWidget(NWID_HORIZONTAL),
@@ -689,8 +680,8 @@ static const NWidgetPart _nested_timetable_widgets[] = {
 	EndContainer(),
 };
 
-static const WindowDesc _timetable_desc(
-	WDP_AUTO, 400, 130,
+static WindowDesc _timetable_desc(
+	WDP_AUTO, "view_vehicle_timetable", 400, 130,
 	WC_VEHICLE_TIMETABLE, WC_VEHICLE_VIEW,
 	WDF_CONSTRUCTION,
 	_nested_timetable_widgets, lengthof(_nested_timetable_widgets)
