@@ -1,4 +1,4 @@
-/* $Id: transparency_gui.cpp 24900 2013-01-08 22:46:42Z planetmaker $ */
+/* $Id: transparency_gui.cpp 25290 2013-05-26 19:25:01Z frosch $ */
 
 /*
  * This file is part of OpenTTD.
@@ -28,9 +28,9 @@ byte _display_opt; ///< What do we want to draw/do?
 class TransparenciesWindow : public Window
 {
 public:
-	TransparenciesWindow(const WindowDesc *desc, int window_number) : Window()
+	TransparenciesWindow(WindowDesc *desc, int window_number) : Window(desc)
 	{
-		this->InitNested(desc, window_number);
+		this->InitNested(window_number);
 	}
 
 	virtual void OnPaint()
@@ -102,7 +102,7 @@ public:
 		}
 	}
 
-	virtual Point OnInitialPosition(const WindowDesc *desc, int16 sm_width, int16 sm_height, int window_number)
+	virtual Point OnInitialPosition(int16 sm_width, int16 sm_height, int window_number)
 	{
 		Point pt = GetToolbarAlignedWindowPosition(sm_width);
 		pt.y += 2 * (sm_height - this->GetWidget<NWidgetBase>(WID_TT_BUTTONS)->current_y);
@@ -146,8 +146,8 @@ static const NWidgetPart _nested_transparency_widgets[] = {
 	EndContainer(),
 };
 
-static const WindowDesc _transparency_desc(
-	WDP_MANUAL, 0, 0,
+static WindowDesc _transparency_desc(
+	WDP_MANUAL, "toolbar_transparency", 0, 0,
 	WC_TRANSPARENCY_TOOLBAR, WC_NONE,
 	0,
 	_nested_transparency_widgets, lengthof(_nested_transparency_widgets)
