@@ -1,4 +1,4 @@
-/* $Id: tunnelbridge_cmd.cpp 25241 2013-05-13 19:22:08Z rubidium $ */
+/* $Id: tunnelbridge_cmd.cpp 26277 2014-01-26 13:50:10Z frosch $ */
 
 /*
  * This file is part of OpenTTD.
@@ -680,7 +680,7 @@ CommandCost CmdBuildTunnel(TileIndex start_tile, DoCommandFlag flags, uint32 p1,
 	switch (transport_type) {
 		case TRANSPORT_ROAD: cost.AddCost((tiles + 2) * _price[PR_BUILD_ROAD] * 2); break;
 		case TRANSPORT_RAIL: cost.AddCost((tiles + 2) * RailBuildCost(railtype)); break;
-		default: break;
+		default: NOT_REACHED();
 	}
 
 	if (flags & DC_EXEC) {
@@ -1223,10 +1223,9 @@ static void DrawTile_TunnelBridge(TileInfo *ti)
 		/* HACK Wizardry to convert the bridge ramp direction into a sprite offset */
 		base_offset += (6 - tunnelbridge_direction) % 4;
 
-		if (ti->tileh == SLOPE_FLAT) base_offset += 4; // sloped bridge head
-
 		/* Table number BRIDGE_PIECE_HEAD always refers to the bridge heads for any bridge type */
 		if (transport_type != TRANSPORT_WATER) {
+			if (ti->tileh == SLOPE_FLAT) base_offset += 4; // sloped bridge head
 			psid = &GetBridgeSpriteTable(GetBridgeType(ti->tile), BRIDGE_PIECE_HEAD)[base_offset];
 		} else {
 			psid = _aqueduct_sprites + base_offset;
@@ -1564,6 +1563,8 @@ static void GetTileDesc_TunnelBridge(TileIndex tile, TileDesc *td)
 				td->rail_speed = spd;
 			}
 		}
+	} else if (tt == TRANSPORT_ROAD && !IsTunnel(tile)) {
+		td->road_speed = GetBridgeSpec(GetBridgeType(tile))->speed;
 	}
 }
 
